@@ -22,18 +22,21 @@
 #include <language/codecompletion/codecompletionmodel.h>
 #include <language/duchain/types/functiontype.h>
 #include <language/duchain/aliasdeclaration.h>
+#include <language/duchain/types/containertypes.h>
+#include <shell/partcontroller.h>
 
 #include <KTextEditor/View>
 #include <KTextEditor/Document>
 #include <KLocalizedString>
 
 #include "duchain/navigation/navigationwidget.h"
-#include "types/variablelengthcontainer.h"
 #include "codecompletion/helpers.h"
 #include "declaration.h"
 #include "declarations/functiondeclaration.h"
 #include "duchain/helpers.h"
 
+#include <QDebug>
+#include "../codecompletiondebug.h"
 
 using namespace KDevelop;
 using namespace KTextEditor;
@@ -44,7 +47,7 @@ FunctionDeclarationCompletionItem::FunctionDeclarationCompletionItem(Declaration
     : PythonDeclarationCompletionItem(decl, context)
     , m_atArgument(-1)
     , m_depth(0)
-    , m_isImportItem(false)
+    , m_doNotCall(false)
 {
 
 }
@@ -71,8 +74,8 @@ int FunctionDeclarationCompletionItem::argumentHintDepth() const
 
 QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const
 {
-    FunctionDeclaration* dec = dynamic_cast<FunctionDeclaration*>(m_declaration.data());
     DUChainReadLocker lock;
+    FunctionDeclaration* dec = dynamic_cast<FunctionDeclaration*>(m_declaration.data());
     switch ( role ) {
         case Qt::DisplayRole: {
             if ( ! dec ) {
@@ -81,13 +84,14 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
             if ( index.column() == KDevelop::CodeCompletionModel::Arguments ) {
                 if (FunctionType::Ptr functionType = dec->type<FunctionType>()) {
                     QString ret;
-                    createArgumentList(dec, ret, 0, 0, ( m_depth > 0 ) );
-                    return ret.replace("__kdevpythondocumentation_builtin_", "");
+                    createArgumentList(dec, ret, 0, 0, false);
+                    return ret;
                 }
             }
             if ( index.column() == KDevelop::CodeCompletionModel::Prefix ) {
-                if ( FunctionType::Ptr type = dec->type<FunctionType>() ) {
-                    return i18n("function") + " -> " + type->returnType()->toString().replace("__kdevpythondocumentation_builtin_", "");
+                FunctionType::Ptr type = dec->type<FunctionType>();
+                if ( type && type->returnType() ) {
+                    return i18n("function") + " -> " + type->returnType()->toString();
                 }
             }
             break;
@@ -103,10 +107,10 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
                 QString ret;
                 QList<QVariant> highlight;
                 if ( atArgument() ) {
-                    createArgumentList(dec, ret, &highlight, atArgument());
+                    createArgumentList(dec, ret, &highlight, atArgument(), false);
                 }
                 else {
-                    createArgumentList(dec, ret, 0);
+                    createArgumentList(dec, ret, 0, false);
                 }
                 return QVariant(highlight);
             }
@@ -114,7 +118,7 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
         case KDevelop::CodeCompletionModel::MatchQuality: {
             if (    m_typeHint == PythonCodeCompletionContext::IterableRequested
                  && dec && dec->type<FunctionType>()
-                 && dynamic_cast<VariableLengthContainer*>(dec->type<FunctionType>()->returnType().unsafeData()) )
+                 && dynamic_cast<ListType*>(dec->type<FunctionType>()->returnType().data()) )
             {
                 return 2 + PythonDeclarationCompletionItem::data(index, role, model).toInt();
             }
@@ -124,44 +128,56 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
     return Python::PythonDeclarationCompletionItem::data(index, role, model);
 }
 
-void FunctionDeclarationCompletionItem::setIsImportItem(bool isImportItem)
+void FunctionDeclarationCompletionItem::setDoNotCall(bool doNotCall)
 {
-    m_isImportItem = isImportItem;
+    m_doNotCall = doNotCall;
 }
 
-void FunctionDeclarationCompletionItem::executed(KTextEditor::Document* document, const KTextEditor::Range& word)
+void FunctionDeclarationCompletionItem::executed(KTextEditor::View* view, const KTextEditor::Range& word)
 {
-    kDebug() << "FunctionDeclarationCompletionItem executed";
-    DeclarationPointer resolvedDecl(Helper::resolveAliasDeclaration(declaration().data()));
+    qCDebug(KDEV_PYTHON_CODECOMPLETION) << "FunctionDeclarationCompletionItem executed";
+    KTextEditor::Document* document = view->document();
+    auto resolvedDecl = Helper::resolveAliasDeclaration(declaration().data());
     DUChainReadLocker lock;
-    QPair<FunctionDeclarationPointer, bool> fdecl = Helper::functionDeclarationForCalledDeclaration(resolvedDecl);
+    auto functionDecl = Helper::functionForCalled(resolvedDecl).declaration;
     lock.unlock();
-    if ( ! fdecl.first && (! resolvedDecl || ! resolvedDecl->abstractType()
+    if ( ! functionDecl && (! resolvedDecl || ! resolvedDecl->abstractType()
                            || resolvedDecl->abstractType()->whichType() != AbstractType::TypeStructure) ) {
-        kError() << "ERROR: could not get declaration data, not executing completion item!";
+        qCritical(KDEV_PYTHON_CODECOMPLETION) << "ERROR: could not get declaration data, not executing completion item!";
         return;
     }
     QString suffix = "()";
     KTextEditor::Range checkPrefix(word.start().line(), 0, word.start().line(), word.start().column());
     KTextEditor::Range checkSuffix(word.end().line(), word.end().column(), word.end().line(), document->lineLength(word.end().line()));
-    if ( m_isImportItem || document->text(checkSuffix).trimmed().startsWith('(')
+    if ( m_doNotCall || document->text(checkSuffix).trimmed().startsWith('(')
          || document->text(checkPrefix).trimmed().endsWith('@')
-         || (fdecl.first && Helper::findDecoratorByName(fdecl.first.data(), QLatin1String("property"))) )
+         || (functionDecl && Helper::findDecoratorByName(functionDecl, QLatin1String("property"))) )
     {
         // don't insert brackets if they're already there,
         // the item is a decorator, or if it's an import item.
-        suffix = "";
+        suffix.clear();
     }
     // place cursor behind bracktes by default
     int skip = 2;
-    if ( fdecl.first && fdecl.first->type<FunctionType>()->arguments().length() != 0 ) {
-        // place cursor in brackets if there's parameters
-        skip = 1;
+    if ( functionDecl ) {
+        bool needsArguments = false;
+        int argumentCount = functionDecl->type<FunctionType>()->arguments().length();
+        if ( functionDecl->context()->type() == KDevelop::DUContext::Class ) {
+            // it's a member function, so it has the implicit self
+            // TODO static methods
+            needsArguments = argumentCount > 1;
+        }
+        else {
+            // it's a free function
+            needsArguments = argumentCount > 0;
+        }
+        if ( needsArguments ) {
+            // place cursor in brackets if there's parameters
+            skip = 1;
+        }
     }
     document->replaceText(word, declaration()->identifier().toString() + suffix);
-    if ( View* view = document->activeView() ) {
-        view->setCursorPosition( Cursor(word.end().line(), word.end().column() + skip) );
-    }
+    view->setCursorPosition( Cursor(word.end().line(), word.end().column() + skip) );
 }
 
 FunctionDeclarationCompletionItem::~FunctionDeclarationCompletionItem() { }
