@@ -37,6 +37,13 @@
 #include <QStandardPaths>
 #include "debuggerdebug.h"
 
+#ifdef Q_OS_WIN
+#include <windows.h>
+#define INTERRUPT_DEBUGGER GenerateConsoleCtrlEvent(CTRL_C_EVENT, m_debuggerProcess->processId())
+#else
+#define INTERRUPT_DEBUGGER kill(m_debuggerProcess->pid(), SIGINT)
+#endif
+
 using namespace KDevelop;
 
 static QByteArray debuggerPrompt = "__KDEVPYTHON_DEBUGGER_PROMPT";
@@ -84,10 +91,10 @@ void DebugSession::start()
     m_debuggerProcess->setOutputChannelMode(KProcess::SeparateChannels);
     m_debuggerProcess->blockSignals(true);
     m_debuggerProcess->setWorkingDirectory(m_workingDirectory.path());
-    connect(m_debuggerProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(dataAvailable()));
+    connect(m_debuggerProcess, &QProcess::readyReadStandardOutput, this, &DebugSession::dataAvailable);
     connect(m_debuggerProcess, SIGNAL(finished(int)), this, SLOT(debuggerQuit(int)));
-    connect(this, SIGNAL(debuggerReady()), SLOT(checkCommandQueue()));
-    connect(this, SIGNAL(commandAdded()), SLOT(checkCommandQueue()));
+    connect(this, &DebugSession::debuggerReady, this, &DebugSession::checkCommandQueue);
+    connect(this, &DebugSession::commandAdded, this, &DebugSession::checkCommandQueue);
     m_debuggerProcess->start();
     m_debuggerProcess->waitForStarted();
     auto dir = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
@@ -133,7 +140,7 @@ void DebugSession::dataAvailable()
         data = data.mid(0, data.length() - debuggerPrompt.length());
     }
     
-    // scan the data, and seperate program output from debugger output
+    // scan the data, and separate program output from debugger output
     int len = data.length();
     int delimiterSkip = debuggerOutputEnd.length();
     int i = 0;
@@ -156,6 +163,8 @@ void DebugSession::dataAvailable()
                                                     << "  " + i18n("You can now inspect the status of the program after it exited.")
                                                     << "  " + i18n("The debugger will silently stop when the next command is triggered.")
                                                     << "*****");
+                InternalPdbCommand* cmd = new InternalPdbCommand(0, 0, "import __kdevpython_debugger_utils\n");
+                addCommand(cmd);
             }
         }
         else if ( m_inDebuggerData == 0 ) {
@@ -345,7 +354,7 @@ void DebugSession::run()
 
 void DebugSession::interruptDebugger()
 {
-    kill(m_debuggerProcess->pid(), SIGINT);
+    INTERRUPT_DEBUGGER;
     updateLocation();
     setState(PausedState);
 }
@@ -400,7 +409,7 @@ void DebugSession::runImmediately(const QString& cmd)
         m_nextNotifyMethod = 0;
         m_nextNotifyObject.clear(); // TODO is this correct?
         qCDebug(KDEV_PYTHON_DEBUGGER) << "interrupting debugger";
-        kill(m_debuggerProcess->pid(), SIGINT);
+        INTERRUPT_DEBUGGER;
         write(cmd.toUtf8());
         write("continue\n");
         updateLocation();
