@@ -279,7 +279,7 @@ void ContextBuilder::openContextForClassDefinition(ClassDefinitionAst* node)
     if ( start.line > node->startLine ) {
         start = CursorInRevision(node->startLine + 1, 0);
     }
-    RangeInRevision range(start, CursorInRevision(endLine + 1, 0));
+    RangeInRevision range(start, CursorInRevision(endLine, 0));
     DUChainWriteLocker lock;
     openContext(node, range, DUContext::Class, node->name);
     currentContext()->setLocalScopeIdentifier(identifierForNode(node->name));
@@ -322,7 +322,7 @@ void ContextBuilder::visitCode(CodeAst* node) {
 QPair<QUrl, QStringList> ContextBuilder::findModulePath(const QString& name, const QUrl& currentDocument)
 {
     QStringList nameComponents = name.split(".");
-    QList<QUrl> searchPaths;
+    QVector<QUrl> searchPaths;
     if ( name.startsWith('.') ) {
         /* To take care for imports like "from ....xxxx.yyy import zzz"
          * we need to take current doc path and run "cd .." enough times
@@ -347,7 +347,7 @@ QPair<QUrl, QStringList> ContextBuilder::findModulePath(const QString& name, con
     // Loop over all the name components, and find matching folders or files.
     QDir tmp;
     QStringList leftNameComponents;
-    foreach ( QUrl currentPath, searchPaths ) {
+    foreach ( const QUrl& currentPath, searchPaths ) {
         tmp.setPath(currentPath.toLocalFile());
         leftNameComponents = nameComponents;
         foreach ( QString component, nameComponents ) {
@@ -400,29 +400,26 @@ RangeInRevision ContextBuilder::rangeForArgumentsContext(FunctionDefinitionAst*
 {
     auto start = node->name->range().end();
     auto end = start;
-    if ( node->arguments->kwarg ) {
-        end = node->arguments->kwarg->range().end();
+    auto args = node->arguments;
+    if ( args->kwarg ) {
+        end = args->kwarg->range().end();
     }
-    else if ( node->arguments->vararg ) {
-        end = node->arguments->vararg->range().end();
+    else if ( args->vararg &&
+         ( args->arguments.isEmpty() ||
+           ! args->vararg->appearsBefore(args->arguments.last())) ) {
+        end = args->vararg->range().end();
     }
-    if ( ! node->arguments->arguments.isEmpty() && node->arguments->vararg ) {
-        if ( node->arguments->vararg->appearsBefore(node->arguments->arguments.last()) ) {
-            end = node->arguments->arguments.last()->range().end();
-        }
-    }
-    else if ( ! node->arguments->arguments.isEmpty() ) {
-        end = node->arguments->arguments.last()->range().end();
+    else if ( ! args->arguments.isEmpty() ) {
+        end = args->arguments.last()->range().end();
     }
 
-    if ( ! node->arguments->defaultValues.isEmpty() ) {
-        end = qMax<KTextEditor::Cursor>(node->arguments->defaultValues.last()->range().end(), end);
+    if ( ! args->defaultValues.isEmpty() ) {
+        end = qMax<KTextEditor::Cursor>(args->defaultValues.last()->range().end(), end);
     }
 
     RangeInRevision range(start.line(), start.column(), end.line(), end.column());
-    // make the range contain the closing and opening parentheses
-    range.start.column -= 1;
-    range.end.column += 1;
+    range.start.column += 1; // Don't include end of name.
+    range.end.column += 1; // Include end parenthesis (unless spaces...)
     return range;
 }
 
