@@ -1,7 +1,8 @@
 /*****************************************************************************
  * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
  * Copyright 2007 Andreas Pakulat <apaku@gmx.de>                             *
- * Copyright 2010-2013 Sven Brauch <svenbrauch@googlemail.com>               *
+ * Copyright 2010-2016 Sven Brauch <svenbrauch@googlemail.com>               *
+ * Copyright 2016 Francis Herne <mail@flherne.uk>                            *
  *                                                                           *
  * This program is free software; you can redistribute it and/or             *
  * modify it under the terms of the GNU General Public License as            *
@@ -92,8 +93,7 @@ ReferencedTopDUContext DeclarationBuilder::build(const IndexedString& url, Ast*
     // which are called before they are defined (which is easily possible, due to python's dynamic nature).
     if ( ! m_prebuilding ) {
         qCDebug(KDEV_PYTHON_DUCHAIN) << "building, but running pre-builder first";
-        DeclarationBuilder* prebuilder = new DeclarationBuilder(editor());
-        prebuilder->m_ownPriority = m_ownPriority;
+        DeclarationBuilder* prebuilder = new DeclarationBuilder(editor(), m_ownPriority);
         prebuilder->m_currentlyParsedDocument = currentlyParsedDocument();
         prebuilder->setPrebuilding(true);
         prebuilder->m_futureModificationRevision = m_futureModificationRevision;
@@ -166,16 +166,6 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Ast* node,
     }
 }
 
-template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier* node, RangeInRevision range,
-                                                                     AbstractType::Ptr type, VisitVariableFlags flags)
-{
-    Ast pseudo;
-    pseudo.startLine = range.start.line; pseudo.startCol = range.start.column;
-    pseudo.endLine = range.end.line; pseudo.endCol = range.end.column;
-    T* result = visitVariableDeclaration<T>(node, &pseudo, nullptr, type, flags);
-    return result;
-}
-
 QList< Declaration* > DeclarationBuilder::existingDeclarationsForNode(Identifier* node)
 {
     QList<Declaration*> existingDeclarations = currentContext()->findDeclarations(
@@ -224,7 +214,7 @@ template<typename T> QList<Declaration*> DeclarationBuilder::reopenFittingDeclar
         // been encountered yet in this pass
         bool reallyEncountered = wasEncountered(d) && ! m_scheduledForDeletion.contains(d);
         bool invalidType = false;
-        if ( d && d->abstractType() && mustFitType != NoTypeRequired ) {
+        if ( d->abstractType() && mustFitType != NoTypeRequired ) {
             invalidType = ( ( d->isFunctionDeclaration() ) != ( mustFitType == FunctionDeclarationType ) );
             if ( ! invalidType ) {
                 invalidType = ( ( dynamic_cast<AliasDeclaration*>(d) != 0 ) != ( mustFitType == AliasDeclarationType ) );
@@ -249,7 +239,6 @@ template<typename T> QList<Declaration*> DeclarationBuilder::reopenFittingDeclar
     return remainingDeclarations;
 }
 
-typedef QPair<Declaration*, int> p;
 template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier* node, Ast* originalAst, Declaration* previous,
                                                                      AbstractType::Ptr type, VisitVariableFlags flags)
 {
@@ -396,75 +385,10 @@ void DeclarationBuilder::visitWithItem(WithItemAst* node)
 
 void DeclarationBuilder::visitFor(ForAst* node)
 {
-    ExpressionVisitor v(currentContext());
-    v.visitNode(node->iterator);
-    auto possibleIterators = Helper::filterType<ListType>(v.lastType(),
-        [](AbstractType::Ptr type) {
-            auto container = type.cast<ListType>();
-            return container && container->contentType();
-        }
-    );
-    if ( node->target->astType == Ast::NameAstType ) {
-        // In case the iterator variable is a Name ("for x in range(3)"), just create a declaration for it.
-        // The following code tries to figure out the type of "x" from the object that is being iterated over.
-        auto iteratorType = Helper::foldTypes<ListType::Ptr>(possibleIterators,
-            [](const ListType::Ptr& p) {
-                return p->contentType().abstractType();
-            }
-        );
-        // otherwise, no list type whatsoever was available for the iterator list, so just display "mixed".
-
-        // Create the variable declaration for the iterator variable with the type that has been determined.
-        visitVariableDeclaration<Declaration>(node->target, 0, iteratorType);
-    }
-    else if ( node->target->astType == Ast::TupleAstType ) {
-        // If the target is a tuple ("for x, y, z in ..."), multiple variables must be declared.
-        // For now, types of those variables will only be determined if the iterator is a list of tuples.
-        QList<ExpressionAst*> targetElements = targetsOfAssignment(QList<ExpressionAst*>() << node->target);
-        int targetElementsCount = targetElements.count();
-        QList<IndexedContainer::Ptr> gatherFromTuples;
-        for ( auto container : possibleIterators ) {
-            AbstractType::Ptr contentType = container->contentType().abstractType();
-            gatherFromTuples = Helper::filterType<IndexedContainer>(contentType,
-                // find all IndexedContainer entries which have the right number of entries
-                [targetElementsCount](AbstractType::Ptr type) {
-                    IndexedContainer::Ptr indexed = type.cast<IndexedContainer>();
-                    return indexed && indexed->typesCount() == targetElementsCount;
-                }
-            );
-        }
-
-        // Now, iterate over all possible tuples, and extract their types
-        int i = 0;
-        QList<AbstractType::Ptr> targetTypes;
-        foreach ( IndexedContainer::Ptr tuple, gatherFromTuples ) {
-            for ( int j = 0; j < tuple->typesCount(); j++ ) {
-                if ( i == 0 ) {
-                    targetTypes.append(tuple->typeAt(j).abstractType());
-                }
-                else {
-                    targetTypes[j] = Helper::mergeTypes(targetTypes[j], tuple->typeAt(j).abstractType());
-                }
-            }
-            i++;
-        }
-
-        short atElement = 0;
-        bool haveTypeInformation = ! targetTypes.isEmpty();
-        Q_ASSERT( ! haveTypeInformation || targetTypes.length() == targetElementsCount );
-        foreach ( ExpressionAst* tupleMember, targetElements ) {
-            if ( tupleMember->astType == Ast::NameAstType ) {
-                AbstractType::Ptr newType;
-                if ( haveTypeInformation ) {
-                    newType = targetTypes.at(atElement);
-                }
-                else {
-                    newType = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
-                }
-                visitVariableDeclaration<Declaration>(tupleMember, 0, newType);
-            }
-            ++atElement;
-        }
+    if ( node->iterator ) {
+        ExpressionVisitor v(currentContext());
+        v.visitNode(node->iterator);
+        assignToUnknown(node->target, Helper::contentOfIterable(v.lastType(), topContext()));
     }
     Python::ContextBuilder::visitFor(node);
 }
@@ -549,41 +473,32 @@ void DeclarationBuilder::visitImportFrom(ImportFromAst* node)
     }
 }
 
+void spoofNodePosition(Ast* node, const CursorInRevision& pos) {
+    // Ridiculous hack, see next comment.
+    node->startLine = node->endLine = pos.line;
+    node->startCol = node->endCol = pos.column - 1;
+    if (node->astType == Ast::TupleAstType) {
+        //  Recursion to bodge all declarations, e.g.
+        //  [x + y * z for x, (y, z) in foo]
+        foreach(auto elem, static_cast<TupleAst*>(node)->elements) {
+            spoofNodePosition(elem, pos);
+        }
+    }
+}
+
 void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
 {
     Python::AstDefaultVisitor::visitComprehension(node);
     // make the declaration zero chars long; it must appear at the beginning of the context,
     // because it is actually used *before* its real declaration: [foo for foo in bar]
     // The DUChain doesn't like this, so for now, the declaration is at the opening bracket,
-    // and both other occurences are uses of that declaration.
-    // TODO add a special case to the usebuilder to display the second occurence as a declaration
-    RangeInRevision declarationRange(currentContext()->range().start, currentContext()->range().start);
-    declarationRange.end.column -= 1;
-    declarationRange.start.column -= 1;
-    
-    AbstractType::Ptr targetType(new IntegralType(IntegralType::TypeMixed));
-    if ( node->iterator ) {
-        // try to find the type of the object being iterated over, for guessing the
-        // type of the iterator variable
-        ExpressionVisitor v(currentContext());
-        v.visitNode(node->iterator);
-        if ( auto container = ListType::Ptr::dynamicCast(v.lastType()) ) {
-            targetType = container->contentType().abstractType();
-        }
-    }
-    
-    // create variable declarations for the iterator variable(s)
-    if ( node->target->astType == Ast::NameAstType ) {
-        visitVariableDeclaration<Declaration>(static_cast<NameAst*>(node->target)->identifier, declarationRange, targetType);
-    }
-    if ( node->target->astType == Ast::TupleAstType ) {
-        foreach ( ExpressionAst* tupleElt, static_cast<TupleAst*>(node->target)->elements ) {
-            if ( tupleElt->astType == Ast::NameAstType ) {
-                NameAst* n = static_cast<NameAst*>(tupleElt);
-                visitVariableDeclaration<Declaration>(n->identifier, declarationRange);
-            }
-        }
-    }
+    // and both other occurrences are uses of that declaration.
+    // TODO add a special case to the usebuilder to display the second occurrence as a declaration
+    spoofNodePosition(node->target, currentContext()->range().start);
+
+    ExpressionVisitor v(currentContext());
+    v.visitNode(node->iterator);
+    assignToUnknown(node->target, Helper::contentOfIterable(v.lastType(), topContext()));
 }
 
 void DeclarationBuilder::visitImport(ImportAst* node)
@@ -945,6 +860,12 @@ void DeclarationBuilder::visitLambda(LambdaAst* node)
     foreach ( ArgAst* argument, node->arguments->arguments ) {
         visitVariableDeclaration<Declaration>(argument->argumentName);
     }
+    if (node->arguments->vararg) {
+        visitVariableDeclaration<Declaration>(node->arguments->vararg->argumentName);
+    }
+    if (node->arguments->kwarg) {
+        visitVariableDeclaration<Declaration>(node->arguments->kwarg->argumentName);
+    }
     closeContext();
 }
 
@@ -958,7 +879,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
     if ( ! container || ! function ) {
         return;
     }
-    // Don't to updates to pre-defined functions.
+    // Don't do updates to pre-defined functions.
     if ( ! v.lastDeclaration() || v.lastDeclaration()->topContext()->url() == IndexedString(Helper::getDocumentationFile()) ) {
         return;
     }
@@ -1006,68 +927,85 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
             v.lastDeclaration()->setType(container);
         }
     };
-
-    foreach ( const QString& key, items.keys() ) {
-        if ( Helper::docstringContainsHint(function.data(), key, &args) ) {
-            items[key]();
+    auto docstring = function->comment();
+    if ( ! docstring.isEmpty() ) {
+        foreach ( const auto& key, items.keys() ) {
+            if ( Helper::docstringContainsHint(docstring, key, &args) ) {
+                items[key]();
+            }
         }
     }
 }
 
-void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer function)
+void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer called)
 {
     DUChainReadLocker lock;
-    QPair<FunctionDeclaration::Ptr, bool> called = Helper::functionDeclarationForCalledDeclaration(function);
-    FunctionDeclaration::Ptr lastFunctionDeclaration = called.first;
-    bool isConstructor = called.second;
+    auto funcInfo = Helper::functionForCalled(called.data());
+    auto function = funcInfo.declaration;
 
-    if ( ! lastFunctionDeclaration ) {
+    if ( ! function ) {
         return;
     }
-    if ( lastFunctionDeclaration->topContext()->url() == IndexedString(Helper::getDocumentationFile()) ) {
+    if ( function->topContext()->url() == IndexedString(Helper::getDocumentationFile()) ) {
         return;
     }
-    DUContext* args = DUChainUtils::getArgumentContext(lastFunctionDeclaration.data());
-    FunctionType::Ptr functiontype = lastFunctionDeclaration->type<FunctionType>();
-    if ( ! args || ! functiontype ) {
+    // Note: within this function:
+    // - 'parameters' refers to the parameters of the function definition.
+    // - 'arguments' refers to the arguments of the function call.
+
+    DUContext* parameterContext = DUChainUtils::getArgumentContext(function);
+    FunctionType::Ptr functionType = function->type<FunctionType>();
+    if ( ! parameterContext || ! functionType ) {
         return;
     }
-    // The declaration which was found is a function declaration, and has a valid arguments list assigned.
-    QVector<Declaration*> parameters = args->localDeclarations();
-    const int specialParamsCount = (lastFunctionDeclaration->vararg() > 0) + (lastFunctionDeclaration->kwarg() > 0);
+    QVector<Declaration*> parameters = parameterContext->localDeclarations();
+    if ( parameters.isEmpty() ) {
+        return;
+    }
+    const int specialParamsCount = (function->vararg() != -1) + (function->kwarg() != -1);
 
     // Look for the "self" in the argument list, the type of that should not be updated.
-    bool hasSelfArgument = false;
-    if ( ( lastFunctionDeclaration->context()->type() == DUContext::Class || isConstructor )
-            && ! parameters.isEmpty() && ! lastFunctionDeclaration->isStatic() )
+    bool hasSelfParam = false;
+    if ( ( function->context()->type() == DUContext::Class || funcInfo.isConstructor )
+            && ! function->isStatic() )
     {
-        // ... unless for some reason the function only has *vararg, **kwarg as arguments
+        // ... unless for some reason the function only has *vararg, **kwarg as parameters
         // (this could happen for example if the method is static but kdev-python does not know,
         // or if the user just made a mistake in his code)
         if ( specialParamsCount < parameters.size() ) {
-            hasSelfArgument = true;
+            hasSelfParam = true;
         }
     }
-    int currentParamIndex = hasSelfArgument;
-    int currentArgumentIndex = 0;
-    int indexInVararg = -1;
-    int paramsAvailable = qMin(functiontype->arguments().length(), parameters.size());
-    int argsAvailable = node->arguments.size();
-    bool atVararg = false;
 
     lock.unlock();
 
+    bool explicitSelfArgument = false;
+    if ( hasSelfParam && ! function->isClassMethod() && node->function->astType == Ast::AttributeAstType ) {
+        // Calling an attribute, e.g. `instance.foo(arg)` or `MyClass.foo(instance, arg)`.
+        ExpressionVisitor valueVisitor(currentContext());
+        valueVisitor.visitNode(static_cast<AttributeAst*>(node->function)->value);
+        if ( valueVisitor.lastDeclaration().dynamicCast<ClassDeclaration>() && valueVisitor.isAlias() ) {
+            // Function is attribute of a class _type_ (not instance), so first arg is used as `self`.
+            explicitSelfArgument = true;
+        }
+    }
+
+    int currentParamIndex = hasSelfParam;
+    int currentArgumentIndex = explicitSelfArgument;
+    int indexInVararg = -1;
+    int paramsAvailable = qMin(functionType->arguments().length(), parameters.size());
+    int argsAvailable = node->arguments.size();
+    bool atVararg = false;
+
     // Iterate over all the arguments, trying to guess the type of the object being
     // passed as an argument, and update the parameter accordingly.
     // Stop if more parameters supplied than possible, and we're not at the vararg.
     for ( ; ( atVararg || currentParamIndex < paramsAvailable ) && currentArgumentIndex < argsAvailable;
-            currentParamIndex++, currentArgumentIndex++ )
+            currentArgumentIndex++ )
     {
-        if ( ! atVararg && currentArgumentIndex == lastFunctionDeclaration->vararg() ) {
-            atVararg = true;
-        }
+        atVararg = atVararg || currentParamIndex == function->vararg(); // Not >=, nonexistent vararg is -1.
 
-        qCDebug(KDEV_PYTHON_DUCHAIN) << currentParamIndex << currentArgumentIndex << atVararg << lastFunctionDeclaration->vararg();
+        qCDebug(KDEV_PYTHON_DUCHAIN) << currentParamIndex << currentArgumentIndex << atVararg << function->vararg();
 
         ExpressionAst* arg = node->arguments.at(currentArgumentIndex);
 
@@ -1083,12 +1021,10 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
         addType->setCreatedBy(topContext(), m_futureModificationRevision);
         closeType();
 
-        // Update the parameter type: change both the type of the function argument,
-        // and the type of the declaration which belongs to that argument
         DUChainWriteLocker wlock;
         if ( atVararg ) {
             indexInVararg++;
-            Declaration* parameter = parameters.at(lastFunctionDeclaration->vararg()+hasSelfArgument);
+            Declaration* parameter = parameters.at(function->vararg());
             IndexedContainer::Ptr varargContainer = parameter->type<IndexedContainer>();
             qCDebug(KDEV_PYTHON_DUCHAIN) << "adding" << addType->toString() << "at position" << indexInVararg;
             if ( ! varargContainer ) continue;
@@ -1107,25 +1043,22 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
             AbstractType::Ptr newType = Helper::mergeTypes(parameters.at(currentParamIndex)->abstractType(),
                                                            addType.cast<AbstractType>());
             // TODO this does not correctly update the types in quickopen! Investigate why.
-            functiontype->removeArgument(currentArgumentIndex + hasSelfArgument);
-            functiontype->addArgument(newType, currentArgumentIndex + hasSelfArgument);
-            lastFunctionDeclaration->setAbstractType(functiontype.cast<AbstractType>());
+            functionType->removeArgument(currentParamIndex);
+            functionType->addArgument(newType, currentParamIndex);
+            function->setAbstractType(functionType.cast<AbstractType>());
             parameters.at(currentParamIndex)->setType(newType);
+            currentParamIndex++;
         }
     }
 
+    // **kwargs is always the last parameter
+    MapType::Ptr kwargsDict;
+    if ( function->kwarg() != -1 ) {
+        kwargsDict = parameters.last()->abstractType().cast<MapType>();
+    }
     lock.unlock();
     DUChainWriteLocker wlock;
-    if ( lastFunctionDeclaration->kwarg() < 0 || parameters.isEmpty() ) {
-        // no kwarg, stop here.
-        return;
-    }
     foreach ( KeywordAst* keyword, node->keywords ) {
-        AbstractType::Ptr param = parameters.last()->abstractType();
-        auto list = param.cast<ListType>();
-        if ( ! list ) {
-            continue;
-        }
         wlock.unlock();
         ExpressionVisitor argumentVisitor(currentContext());
         argumentVisitor.visitNode(keyword->value);
@@ -1133,14 +1066,40 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
             continue;
         }
         wlock.lock();
+        bool matchedNamedParam = false;
         HintedType::Ptr addType = HintedType::Ptr(new HintedType());
-        openType(addType);
-        addType->setType(argumentVisitor.lastType());
-        addType->setCreatedBy(topContext(), m_futureModificationRevision);
-        closeType();
-        list->addContentType<Python::UnsureType>(addType.cast<AbstractType>());
-        parameters.last()->setAbstractType(list.cast<AbstractType>());
+        if ( keyword->argumentName ) {
+            openType(addType);
+            addType->setType(argumentVisitor.lastType());
+            addType->setCreatedBy(topContext(), m_futureModificationRevision);
+            closeType();
+            for (int ip = currentParamIndex; ip < paramsAvailable; ++ip ) {
+                if ( parameters.at(ip)->identifier().toString() != keyword->argumentName->value ) {
+                    continue;
+                }
+                matchedNamedParam = true;
+                auto newType = Helper::mergeTypes(parameters.at(ip)->abstractType(), addType);
+                functionType->removeArgument(ip);
+                functionType->addArgument(newType, ip);
+                parameters.at(ip)->setType(newType);
+            }
+        }
+        else if ( auto unpackedDict = argumentVisitor.lastType().cast<MapType>() ) {
+            // 'keyword is actually an unpacked dict: `foo(**{'a': 12}).
+            openType(addType);
+            addType->setType(unpackedDict->contentType().abstractType());
+            addType->setCreatedBy(topContext(), m_futureModificationRevision);
+            closeType();
+        }
+        else { // Maybe the dict type wasn't loaded yet, or something else happened.
+            continue;
+        }
+        if ( ! matchedNamedParam && kwargsDict ) {
+            kwargsDict->addContentType<Python::UnsureType>(addType);
+            parameters.last()->setAbstractType(kwargsDict);
+        }
     }
+    function->setAbstractType(functionType);
 }
 
 void DeclarationBuilder::visitCall(CallAst* node)
@@ -1175,56 +1134,6 @@ void DeclarationBuilder::visitCall(CallAst* node)
     addArgumentTypeHints(node, functionVisitor.lastDeclaration());
 }
 
-QList<ExpressionAst*> DeclarationBuilder::targetsOfAssignment(QList<ExpressionAst*> targets) const
-{
-    QList<ExpressionAst*> lhsExpressions;
-    foreach ( ExpressionAst* target, targets ) {
-        if ( target->astType == Ast::TupleAstType ) {
-            TupleAst* tuple = static_cast<TupleAst*>(target);
-            foreach ( ExpressionAst* ast, tuple->elements ) {
-                // eventually recursive call, to handle e.g. a, (b, c) = 1, 2, 3 correctly
-                if ( ast->astType == Ast::TupleAstType ) {
-                    lhsExpressions << targetsOfAssignment(QList<ExpressionAst*>() << ast);
-                }
-                else {
-                    // shortcut
-                    lhsExpressions << ast;
-                }
-            }
-        }
-        else {
-            lhsExpressions << target;
-        }
-    }
-    return lhsExpressions;
-}
-
-QVector<DeclarationBuilder::SourceType>
-    DeclarationBuilder::unpackAssignmentSource(const SourceType& source,
-                                               int fillWhenLengthMissing) const
-{
-    QVector<SourceType> sources;
-    if ( const IndexedContainer::Ptr container = source.type.cast<IndexedContainer>() ) {
-        // RHS is a tuple or similar, unpack that.
-        for (int i = 0; i < container->typesCount(); ++i) {
-            sources << SourceType{
-                container->typeAt(i).abstractType(),
-                DeclarationPointer(),
-                false
-            };
-        }
-    }
-    else if ( auto container = ListType::Ptr::dynamicCast(source.type) ) {
-        // RHS is a list or similar, can't tell contents apart.
-        // Return content type * requested length.
-        AbstractType::Ptr content = container->contentType().abstractType();
-        for ( ; fillWhenLengthMissing != 0; fillWhenLengthMissing-- ) {
-            sources << SourceType{ content, DeclarationPointer(), false };
-        }
-    }
-    return sources;
-}
-
 void DeclarationBuilder::assignToName(NameAst* target, const DeclarationBuilder::SourceType& element)
 {
     if ( element.isAlias ) {
@@ -1314,8 +1223,8 @@ void DeclarationBuilder::assignToAttribute(AttributeAst* attrib, const Declarati
     Declaration* attributeDeclaration = nullptr;
     {
         DUChainReadLocker lock;
-        attributeDeclaration = Helper::accessAttribute(parentObjectDeclaration.data(),
-                                                       attrib->attribute->value, currentContext());
+        attributeDeclaration = Helper::accessAttribute(parentObjectDeclaration->abstractType(),
+                                                       attrib->attribute->value, topContext());
     }
 
     if ( ! attributeDeclaration || ! wasEncountered(attributeDeclaration) ) {
@@ -1356,44 +1265,80 @@ void DeclarationBuilder::assignToAttribute(AttributeAst* attrib, const Declarati
     }
 }
 
-void DeclarationBuilder::assignToTuple(TupleAst* tuple, const DeclarationBuilder::SourceType& element) {
-    auto sources = unpackAssignmentSource(element, tuple->elements.length());
-    int ii = 0;
-    bool foundStarred = false;
-    foreach ( ExpressionAst* target, tuple->elements ) {
-        // Fallback, if we ran out of known types.
-        auto source = SourceType{
-            AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)),
-            DeclarationPointer(),
-            false
-        };
-
-        if ( target->astType == Ast::StarredAstType ) {
-            // PEP-3132. `a, *b, c = 1, 2, 3, 4 -> b = [2, 3]`
-            // Starred expression is assigned a list of the values not used by unstarred ones.
-            DUChainReadLocker lock;
-            auto type = ExpressionVisitor::typeObjectForIntegralType<ListType>("list");
-            lock.unlock();
-            if ( !foundStarred ) { // Only allowed once, return unknown list.
-                // Count sources not used by other targets, slurp that many into list.
-                int leftovers = sources.length() - tuple->elements.length() + 1;
-                for ( ; leftovers > 0; leftovers--, ii++ ) {
-                    type->addContentType<Python::UnsureType>(sources.at(ii).type);
+void DeclarationBuilder::tryUnpackType(AbstractType::Ptr sourceType, QVector<AbstractType::Ptr>& outTypes, int starred) {
+    if ( const auto indexed = sourceType.cast<IndexedContainer>() ) {
+        int spare = indexed->typesCount() - outTypes.length();
+        if ( spare < -1 || (starred == -1 && spare != 0) ) {
+            return; // Wrong number of elements to unpack.
+        }
+        for ( int i_out = 0, i_in = 0; i_out < outTypes.length(); ++i_out ) {
+            if ( i_out == starred ) { // PEP-3132. Made into list in assignToTuple().
+                for (; spare >= 0; --spare, ++i_in ) {
+                    auto content = indexed->typeAt(i_in).abstractType();
+                    outTypes[i_out] = Helper::mergeTypes(outTypes.at(i_out), content);
                 }
+            } else {
+                auto content = indexed->typeAt(i_in).abstractType();
+                outTypes[i_out] = Helper::mergeTypes(outTypes.at(i_out), content);
+                ++i_in;
             }
-            source.type = AbstractType::Ptr::staticCast(type);
-            // List is assigned to the child expression.
-            target = static_cast<StarredAst*>(target)->value;
-            foundStarred = true;
         }
-        else if ( ii < sources.length() ) {
-            source = sources.at(ii);
-            ii++;
+    } else {
+        auto content = Helper::contentOfIterable(sourceType, topContext());
+        if ( !Helper::isUsefulType(content) ) {
+            return;
+        }
+        for (auto out = outTypes.begin(); out != outTypes.end(); ++out) {
+            *out = Helper::mergeTypes(*out, content);
         }
-        assignToUnknown(target, source);
     }
 }
 
+void DeclarationBuilder::assignToTuple(TupleAst* tuple, const SourceType& element) {
+    int starred = -1;  // Index (if any) of PEP-3132 starred assignment.
+    for (int ii = 0; ii < tuple->elements.length(); ++ii) {
+         if (tuple->elements.at(ii)->astType == Ast::StarredAstType) {
+            starred = ii;
+            break;
+        }
+    }
+
+    QVector<AbstractType::Ptr> outTypes(tuple->elements.length());
+
+    if ( auto unsure = element.type.cast<UnsureType>() ) {
+        FOREACH_FUNCTION ( const auto& type, unsure->types ) {
+            tryUnpackType(type.abstractType(), outTypes, starred);
+        }
+    } else {
+        tryUnpackType(element.type, outTypes, starred);
+    }
+
+    for (int ii = 0; ii < outTypes.length(); ++ii) {
+        const auto sourceType = outTypes.at(ii);
+        auto target = tuple->elements.at(ii);
+        if ( target->astType == Ast::StarredAstType ) {
+            DUChainReadLocker lock;
+            auto listType = ExpressionVisitor::typeObjectForIntegralType<ListType>("list");
+            lock.unlock();
+            if (listType) {
+                listType->addContentType<Python::UnsureType>(sourceType);
+                assignToUnknown(static_cast<StarredAst*>(target)->value, listType);
+            }
+        } else {
+            assignToUnknown(target, sourceType);
+        }
+    }
+}
+
+void DeclarationBuilder::assignToUnknown(ExpressionAst* target, const AbstractType::Ptr type) {
+    auto source = SourceType{
+        type,
+        DeclarationPointer(),
+        false
+    };
+    assignToUnknown(target, source);
+}
+
 void DeclarationBuilder::assignToUnknown(ExpressionAst* target, const DeclarationBuilder::SourceType& element) {
     // Must be a nicer way to do this.
     if ( target->astType == Ast::TupleAstType ) {
@@ -1448,24 +1393,25 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     dec->clearBaseClasses();
     dec->setClassType(ClassDeclarationData::Class);
 
-    dec->setComment(getDocstring(node->body));
-    
-    // check whether this is a type container (list, dict, ...) or just a "normal" class
-    if ( Helper::docstringContainsHint(dec, "TypeContainer") ) {
-        ListType* container = nullptr;
-        if ( Helper::docstringContainsHint(dec, "hasTypedKeys") ) {
-            container = new MapType();
+    auto docstring = getDocstring(node->body);
+    dec->setComment(docstring);
+    if ( ! docstring.isEmpty() ) {
+        // check whether this is a type container (list, dict, ...) or just a "normal" class
+        if ( Helper::docstringContainsHint(docstring, "TypeContainer") ) {
+            ListType* container = nullptr;
+            if ( Helper::docstringContainsHint(docstring, "hasTypedKeys") ) {
+                container = new MapType();
+            }
+            else {
+                container = new ListType();
+            }
+            type = StructureType::Ptr(container);
         }
-        else {
-            container = new ListType();
+        if ( Helper::docstringContainsHint(docstring, "IndexedTypeContainer") ) {
+            IndexedContainer* container = new IndexedContainer();
+            type = StructureType::Ptr(container);
         }
-        type = StructureType::Ptr(container);
     }
-    if ( Helper::docstringContainsHint(dec, "IndexedTypeContainer") ) {
-        IndexedContainer* container = new IndexedContainer();
-        type = StructureType::Ptr(container);
-    }
-    
     lock.unlock();
     foreach ( ExpressionAst* c, node->baseClasses ) {
         // Iterate over all the base classes, and add them to the duchain.
@@ -1578,28 +1524,11 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
     
     lock.unlock();
     visitDecorators<FunctionDeclaration>(node->decorators, dec);
-    visitFunctionArguments(node);
-    
-    lock.lock();
     const bool isStatic = Helper::findDecoratorByName<FunctionDeclaration>(dec, "staticmethod");
     const bool isClassMethod = Helper::findDecoratorByName<FunctionDeclaration>(dec, "classmethod");
     dec->setStatic(isStatic);
-    
-    // If this is a member function, set the type of the first argument (the "self") to be
-    // an instance of the class.
-    // this must be done here, because the type of self must be known when parsing the body
-    if ( eventualParentDeclaration && currentType<FunctionType>()->arguments().length()
-            && currentContext()->type() == DUContext::Class && ! isStatic )
-    {
-        if ( dec->vararg() != -1 ) {
-            dec->setVararg(dec->vararg() - 1);
-        }
-        if ( dec->kwarg() != -1 ) {
-            dec->setKwarg(dec->kwarg() - 1);
-        }
-    }
-
-    lock.unlock();
+    dec->setClassMethod(isClassMethod);
+    visitFunctionArguments(node);
     visitFunctionBody(node);
     lock.lock();
     
@@ -1841,7 +1770,6 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
     }
     FunctionDeclaration* workingOnDeclaration = static_cast<FunctionDeclaration*>(Helper::resolveAliasDeclaration(currentDeclaration()));
     workingOnDeclaration->clearDefaultParameters();
-    const bool isClassMethod = Helper::findDecoratorByName<FunctionDeclaration>(workingOnDeclaration, "classmethod");
     if ( ! hasCurrentType() || ! currentType<FunctionType>() ) {
         return;
     }
@@ -1865,7 +1793,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
 
         // Create a variable declaration for the parameter, to be used in the function body.
         Declaration* paramDeclaration = nullptr;
-        if ( currentIndex == 1 && isClassMethod ) {
+        if ( currentIndex == 1 && workingOnDeclaration->isClassMethod() ) {
             DUChainWriteLocker lock;
             AliasDeclaration* decl = eventuallyReopenDeclaration<AliasDeclaration>(arg->argumentName,
                                                                                    arg, AliasDeclarationType);
@@ -1873,6 +1801,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
                 qCDebug(KDEV_PYTHON_DUCHAIN) << "setting declaration:" << m_currentClassType->declaration(topContext())->toString();
                 decl->setAliasedDeclaration(m_currentClassType->declaration(currentContext()->topContext()));
             }
+            closeDeclaration();
             paramDeclaration = decl;
         }
         else {
@@ -1905,7 +1834,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
         }
 
         qCDebug(KDEV_PYTHON_DUCHAIN) << "is first:" << isFirst << hasCurrentDeclaration() << currentDeclaration();
-        if ( isFirst && hasCurrentDeclaration() && currentContext() && currentContext()->parentContext() ) {
+        if ( isFirst && ! workingOnDeclaration->isStatic() && currentContext() && currentContext()->parentContext() ) {
             DUChainReadLocker lock;
             if ( currentContext()->parentContext()->type() == DUContext::Class ) {
                 argumentType = m_currentClassType.cast<AbstractType>();
