@@ -91,53 +91,22 @@ void ExpressionVisitor::encounter(AbstractType::Ptr type, DeclarationPointer dec
 
 void ExpressionVisitor::visitAttribute(AttributeAst* node)
 {
-    ExpressionAst* accessingAttributeOf = node->value;
-
     ExpressionVisitor v(this);
-    v.visitNode(accessingAttributeOf);
-    AbstractType::Ptr accessedType = v.lastType();
-    QList<StructureType::Ptr> accessingAttributeOfType = Helper::filterType<StructureType>(accessedType,
-        [](AbstractType::Ptr type) {
-            auto resolved = Helper::resolveAliasType(type);
-            return resolved && resolved->whichType() == AbstractType::TypeStructure;
-        },
-        [](AbstractType::Ptr type) {
-            return Helper::resolveAliasType(type).cast<StructureType>();
-        }
-    );
+    v.visitNode(node->value);
+    setConfident(false);
 
-    // Step 1: Find all matching declarations which are made inside the type of which the accessed object is.
-    // Like, for A.B.C where B is an instance of foo, when processing C, find all properties of foo which are called C.
-    bool haveOneUsefulType = false;
-    Declaration* foundDeclaration = nullptr;
+    // Find a matching declaration which is made inside the type of the accessed object.
+    // Like, for B.C where B is an instance of foo, find a property of foo called C.
     DUChainReadLocker lock;
-    foreach ( StructureType::Ptr current, accessingAttributeOfType ) {
-        if ( Helper::isUsefulType(current.cast<AbstractType>()) ) {
-            haveOneUsefulType = true;
-        }
-        foundDeclaration = Helper::accessAttribute(current->declaration(context()->topContext()),
-                                                   node->attribute->value, context());
-        if ( foundDeclaration ) {
-            break;
-        }
-    }
-    if ( ! haveOneUsefulType ) {
-        setConfident(false);
-    }
+    auto attribute = Helper::accessAttribute(v.lastType(), node->attribute->value, topContext());
 
-    // Step 2: Construct the type of the declaration which was found.
-    if ( foundDeclaration ) {
-        auto d = Helper::resolveAliasDeclaration(foundDeclaration);
-        if ( ! d ) {
-            return encounterUnknown();
-        }
-        bool isAlias =     dynamic_cast<AliasDeclaration*>(foundDeclaration) || d->isFunctionDeclaration()
-                        || dynamic_cast<ClassDeclaration*>(d);
-        encounter(foundDeclaration->abstractType(), DeclarationPointer(foundDeclaration));
-        setLastIsAlias(isAlias);
-    }
-    else {
-        return encounterUnknown();
+    if ( auto resolved = Helper::resolveAliasDeclaration(attribute) ) {
+        encounter(attribute->abstractType(), DeclarationPointer(attribute));
+        setLastIsAlias(dynamic_cast<AliasDeclaration*>(attribute) ||
+                        resolved->isFunctionDeclaration() ||
+                        dynamic_cast<ClassDeclaration*>(resolved));
+    } else {
+        encounterUnknown();
     }
 }
 
@@ -146,78 +115,61 @@ void ExpressionVisitor::visitCall(CallAst* node)
     foreach ( ExpressionAst* c, node->arguments ) {
         AstDefaultVisitor::visitNode(c);
     }
-    
     ExpressionVisitor v(this);
     v.visitNode(node->function);
-    Declaration* actualDeclaration = 0;
-    FunctionType::Ptr unidentifiedFunctionType;
-    if ( ! v.m_isAlias && v.lastType() && v.lastType()->whichType() == AbstractType::TypeFunction ) {
-        unidentifiedFunctionType = v.lastType().cast<FunctionType>();
-    }
-    else if ( ! v.m_isAlias && v.lastType() && v.lastType()->whichType() == AbstractType::TypeStructure ) {
-        // use __call__
-        DUChainReadLocker lock;
-        auto c = v.lastType().cast<StructureType>()->internalContext(topContext());
-        if ( c ) {
-            auto decls = c->findDeclarations(QualifiedIdentifier("__call__"));
-            if ( ! decls.isEmpty() ) {
-                auto decl = dynamic_cast<FunctionDeclaration*>(decls.first());
-                if ( decl ) {
-                    unidentifiedFunctionType = decl->abstractType().cast<FunctionType>();
-                }
-            }
+    auto declaration = Helper::resolveAliasDeclaration(v.lastDeclaration().data());
+    if ( ! v.isAlias() && v.lastType() ) {
+        if ( auto functionType = v.lastType().cast<FunctionType>() ) {
+            encounter(functionType->returnType());
+            return;
+        }
+        if ( auto classType = v.lastType().cast<StructureType>() ) {
+            declaration = classType->declaration(topContext());
         }
     }
-    else {
-        actualDeclaration = v.lastDeclaration().data();
-    }
-    
-    if ( unidentifiedFunctionType ) {
-        encounter(unidentifiedFunctionType->returnType());
+    if ( ! declaration ) {
+        encounterUnknown();
         return;
     }
-    else if ( !actualDeclaration ) {
-        setConfident(false);
-        return encounterUnknown();
-    }
-
+    ClassDeclaration* classDecl = dynamic_cast<ClassDeclaration*>(declaration);
     DUChainReadLocker lock;
-    actualDeclaration = Helper::resolveAliasDeclaration(actualDeclaration);
-    ClassDeclaration* classDecl = dynamic_cast<ClassDeclaration*>(actualDeclaration);
-    QPair<FunctionDeclarationPointer, bool> d = Helper::functionDeclarationForCalledDeclaration(
-                                                DeclarationPointer(actualDeclaration));
-    FunctionDeclaration* funcDecl = d.first.data();
-    bool isConstructor = d.second;
+    auto function = Helper::functionForCalled(declaration, v.isAlias());
     lock.unlock();
 
-    if ( funcDecl && funcDecl->type<FunctionType>() ) {
-        // try to deduce type from a decorator
-        checkForDecorators(node, funcDecl, classDecl, isConstructor);
+    AbstractType::Ptr type;
+    Declaration* decl;
+
+    if ( function.isConstructor && classDecl ) {
+        // Don't use return type from constructor.
+        // It's wrong for builtins, or classes without their own __init__ methods().
+        type = classDecl->abstractType();
+        decl = classDecl;
     }
-    else if ( classDecl ) {
-        return encounter(classDecl->abstractType(), DeclarationPointer(classDecl));
+    else if ( function.declaration && function.declaration->type<FunctionType>() ) {
+        // But do use the return value of normal functions or __call__().
+        type = function.declaration->type<FunctionType>()->returnType();
+        decl = function.declaration;
     }
     else {
-        if ( actualDeclaration ) {
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "Declaraton is not a class or function declaration";
-        }
-        return encounterUnknown();
+        qCDebug(KDEV_PYTHON_DUCHAIN) << "Declaration is not a class or function declaration";
+        encounterUnknown();
+        return;
     }
+    if ( function.declaration ) {
+        auto docstring = function.declaration->comment();
+        if ( ! docstring.isEmpty() ) {
+            // Our documentation data uses special docstrings that override the return type
+            //  of some functions (including constructors).
+            type = docstringTypeOverride(node, type, docstring);
+        }
+    }
+    encounter(type, DeclarationPointer(decl));
 }
 
-void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* funcDecl, ClassDeclaration* classDecl, bool isConstructor)
+AbstractType::Ptr ExpressionVisitor::docstringTypeOverride(
+    CallAst* node, const AbstractType::Ptr normalType, const QString& docstring)
 {
-    AbstractType::Ptr type;
-    Declaration* useDeclaration = nullptr;
-    if ( isConstructor && classDecl ) {
-        type = classDecl->abstractType();
-        useDeclaration = classDecl;
-    }
-    else {
-        type = funcDecl->type<FunctionType>()->returnType();
-        useDeclaration = funcDecl;
-    }
-
+    auto docstringType = normalType;
     auto listOfTuples = [&](AbstractType::Ptr key, AbstractType::Ptr value) {
         auto newType = typeObjectForIntegralType<ListType>("list");
         IndexedContainer::Ptr newContents = typeObjectForIntegralType<IndexedContainer>("tuple");
@@ -248,8 +200,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
         baseTypeVisitor.visitNode(static_cast<AttributeAst*>(node->function)->value);
         if ( auto t = baseTypeVisitor.lastType().cast<ListType>() ) {
             qCDebug(KDEV_PYTHON_DUCHAIN) << "Found container, using type";
-            AbstractType::Ptr newType = t->contentType().abstractType();
-            encounter(newType, DeclarationPointer(useDeclaration));
+            docstringType = t->contentType().abstractType();
             return true;
         }
         return false;
@@ -277,8 +228,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
                 contentType = map->keyType().abstractType();
             }
             newType->addContentType<Python::UnsureType>(contentType);
-            AbstractType::Ptr resultingType = newType.cast<AbstractType>();
-            encounter(resultingType, DeclarationPointer(useDeclaration));
+            docstringType = newType.cast<AbstractType>();
             return true;
         }
         return false;
@@ -295,8 +245,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
         DUChainWriteLocker lock;
         auto intType = typeObjectForIntegralType<AbstractType>("int");
         auto enumerated = enumeratedTypeVisitor.lastType();
-        auto result = listOfTuples(intType, Helper::contentOfIterable(enumerated));
-        encounter(result, DeclarationPointer(useDeclaration));
+        docstringType = listOfTuples(intType, Helper::contentOfIterable(enumerated, topContext()));
         return true;
     };
 
@@ -311,8 +260,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
         DUChainWriteLocker lock;
         if ( auto t = baseTypeVisitor.lastType().cast<MapType>() ) {
             qCDebug(KDEV_PYTHON_DUCHAIN) << "Got container:" << t->toString();
-            auto resultingType = listOfTuples(t->keyType().abstractType(), t->contentType().abstractType());
-            encounter(resultingType, DeclarationPointer(useDeclaration));
+            docstringType = listOfTuples(t->keyType().abstractType(), t->contentType().abstractType());
             return true;
         }
         return false;
@@ -331,7 +279,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
             return false;
         }
         ListType::Ptr realTarget;
-        if ( auto target = ListType::Ptr::dynamicCast(type) ) {
+        if ( auto target = ListType::Ptr::dynamicCast(normalType) ) {
             realTarget = target;
         }
         if ( auto source = ListType::Ptr::dynamicCast(v.lastType()) ) {
@@ -342,7 +290,7 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
             auto newType = ListType::Ptr::staticCast(AbstractType::Ptr(realTarget->clone()));
             Q_ASSERT(newType);
             newType->addContentType<Python::UnsureType>(source->contentType().abstractType());
-            encounter(AbstractType::Ptr::staticCast(newType), DeclarationPointer(useDeclaration));
+            docstringType = AbstractType::Ptr::staticCast(newType);
             return true;
         }
         return false;
@@ -350,35 +298,42 @@ void ExpressionVisitor::checkForDecorators(CallAst* node, FunctionDeclaration* f
 
     foreach ( const QString& currentHint, knownDecoratorHints.keys() ) {
         QStringList arguments;
-        if ( ! Helper::docstringContainsHint(funcDecl, currentHint, &arguments) ) {
+        if ( ! Helper::docstringContainsHint(docstring, currentHint, &arguments) ) {
             continue;
         }
         // If the hint word appears in the docstring, run the evaluation function.
         if ( knownDecoratorHints[currentHint](arguments, currentHint) ) {
             // We indeed found something, so we're done.
-            return;
+            return docstringType;
         }
     }
 
     // if none of the above decorator-finding methods worked, just use the ordinary return type.
-    return encounter(type, DeclarationPointer(useDeclaration));
+    return docstringType;
 }
 
 void ExpressionVisitor::visitSubscript(SubscriptAst* node)
 {
     AstDefaultVisitor::visitNode(node->value);
-    if ( node->slice && node->slice->astType == Ast::IndexAstType ) {
-        DUChainReadLocker lock;
-        auto indexedTypes = Helper::filterType<IndexedContainer>(lastType(), [](AbstractType::Ptr toFilter) {
-            return toFilter.cast<IndexedContainer>();
-        });
-        for ( IndexedContainer::Ptr indexed: indexedTypes ) {
-            // TODO This loop currently only uses the first result, it could construct
-            // an unsure from all the matches.
+
+    auto valueTypes = Helper::filterType<AbstractType>(lastType(), [](AbstractType::Ptr) { return true; });
+    AbstractType::Ptr result(new IntegralType(IntegralType::TypeMixed));
+
+    foreach (const auto& type, valueTypes) {
+        if ( (node->slice->astType != Ast::IndexAstType) &&
+             (type.cast<IndexedContainer>() || type.cast<ListType>()) ) {
+            if ( type.cast<MapType>() ) {
+                continue; // Can't slice dicts.
+            }
+            // Assume that slicing (e.g. foo[3:5]) a tuple/list returns the same type.
+            // TODO: we could do better for some tuple slices.
+            result = Helper::mergeTypes(result, type);
+        }
+        else if ( const auto& indexed = type.cast<IndexedContainer>() ) {
             IndexAst* sliceIndexAst = static_cast<IndexAst*>(node->slice);
             NumberAst* number = nullptr;
             bool invert = false;
-            if ( sliceIndexAst->value && sliceIndexAst->value->astType == Ast::UnaryOperationAstType ) {
+            if ( sliceIndexAst->value->astType == Ast::UnaryOperationAstType ) {
                 // might be -3
                 UnaryOperationAst* unary = static_cast<UnaryOperationAst*>(sliceIndexAst->value);
                 if ( unary->type == Ast::UnaryOperatorSub && unary->operand->astType == Ast::NumberAstType ) {
@@ -395,45 +350,40 @@ void ExpressionVisitor::visitSubscript(SubscriptAst* node)
                     sliceIndex += indexed->typesCount();
                 }
                 if ( sliceIndex < indexed->typesCount() && sliceIndex >= 0 ) {
-                    return encounter(indexed->typeAt(sliceIndex).abstractType());
+                    result = Helper::mergeTypes(result, indexed->typeAt(sliceIndex).abstractType());
+                    continue;
                 }
             }
-            // the exact index is unknown, use unsure
-            return encounter(indexed->asUnsureType().cast<AbstractType>());
+            result = Helper::mergeTypes(result, indexed->asUnsureType());
         }
-        auto variableTypes = Helper::filterType<ListType>(lastType(), [](AbstractType::Ptr toFilter) {
-            return toFilter.cast<ListType>();
-        });
-        if ( ! variableTypes.isEmpty() ) {
-            AbstractType::Ptr result(new IntegralType(IntegralType::TypeMixed));
-            for ( auto variable: variableTypes ) {
-                result = Helper::mergeTypes(result, variable->contentType().abstractType());
+        else if ( const auto& listType = type.cast<ListType>() ) {
+            result = Helper::mergeTypes(result, listType->contentType().abstractType());
+        }
+        else {
+            // Type wasn't one with custom handling, so use return type of __getitem__().
+            DUChainReadLocker lock;
+            static const IndexedIdentifier getitemIdentifier(KDevelop::Identifier("__getitem__"));
+            auto function = Helper::accessAttribute(type, getitemIdentifier, topContext());
+            if ( function && function->isFunctionDeclaration() ) {
+                if ( FunctionType::Ptr functionType = function->type<FunctionType>() ) {
+                    result = Helper::mergeTypes(result, functionType->returnType());
+                }
             }
-            return encounter(result);
         }
     }
+    encounter(result);
+}
 
-    // If that does not work and we have a slice like [3:5], guess it will remain the same type.
-    // That is an approximation we have to make now, should optimally be corrected later.
-    // The reason is that we'd need to parse decorators from the __getitem__ method to support
-    // list/dict/etc properly otherwise, which requires a bit of refactoring first. TODO do this
-    if ( node->slice && node->slice->astType != Ast::IndexAstType ) {
-        return;
+void ExpressionVisitor::visitLambda(LambdaAst* node)
+{
+    AstDefaultVisitor::visitLambda(node);
+    FunctionType::Ptr type(new FunctionType());
+    AbstractType::Ptr mixed(new IntegralType(IntegralType::TypeMixed));
+    for (int ii = 0; ii < node->arguments->arguments.length(); ++ii) {
+        type->addArgument(mixed);
     }
-
-    // Otherwise, try to use __getitem__.
-    ExpressionVisitor v(context());
-    v.visitNode(node->value);
-    DUChainReadLocker lock;
-    Declaration* function = Helper::accessAttribute(v.lastDeclaration().data(), "__getitem__", context());
-    if ( function && function->isFunctionDeclaration() ) {
-        if ( FunctionType::Ptr functionType = function->type<FunctionType>() ) {
-            return encounter(functionType->returnType());
-        }
-    }
-
-    // Otherwise, give up
-    return encounterUnknown();
+    type->setReturnType(lastType());
+    encounter(type);
 }
 
 void ExpressionVisitor::visitList(ListAst* node)
@@ -445,7 +395,13 @@ void ExpressionVisitor::visitList(ListAst* node)
     if ( type ) {
         foreach ( ExpressionAst* content, node->elements ) {
             contentVisitor.visitNode(content);
-            type->addContentType<Python::UnsureType>(contentVisitor.lastType());
+            if ( content->astType == Ast::StarredAstType ) {
+                auto contentType = Helper::contentOfIterable(contentVisitor.lastType(), topContext());
+                type->addContentType<Python::UnsureType>(contentType);
+            }
+            else {
+                type->addContentType<Python::UnsureType>(contentVisitor.lastType());
+            }
         }
     }
     else {
@@ -528,12 +484,16 @@ void ExpressionVisitor::visitTuple(TupleAst* node) {
         foreach ( ExpressionAst* expr, node->elements ) {
             ExpressionVisitor v(this);
             v.visitNode(expr);
-            if ( v.lastType() ) {
+            if ( expr->astType == Ast::StarredAstType ) {
+                // foo = a, *b, c
+                if ( auto unpackedType = v.lastType().cast<IndexedContainer>() ) {
+                    for ( int ii = 0; ii < unpackedType->typesCount(); ++ii ) {
+                        type->addEntry(unpackedType->typeAt(ii).abstractType());
+                    }
+                } // Unpacking something else, do nothing (i.e. assume it was empty).
+            } else {
                 type->addEntry(v.lastType());
             }
-            else {
-                type->addEntry(AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)));
-            }
         }
         encounter(AbstractType::Ptr::staticCast(type));
     }
@@ -565,7 +525,13 @@ void ExpressionVisitor::visitSet(SetAst* node)
     if ( type ) {
         foreach ( ExpressionAst* content, node->elements ) {
             contentVisitor.visitNode(content);
-            type->addContentType<Python::UnsureType>(contentVisitor.lastType());
+            if ( content->astType == Ast::StarredAstType ) {
+                auto contentType = Helper::contentOfIterable(contentVisitor.lastType(), topContext());
+                type->addContentType<Python::UnsureType>(contentType);
+            }
+            else {
+                type->addContentType<Python::UnsureType>(contentVisitor.lastType());
+            }
         }
     }
     encounter(AbstractType::Ptr::staticCast(type));
@@ -579,13 +545,19 @@ void ExpressionVisitor::visitDict(DictAst* node)
     ExpressionVisitor contentVisitor(this);
     ExpressionVisitor keyVisitor(this);
     if ( type ) {
-        foreach ( ExpressionAst* content, node->values ) {
-            contentVisitor.visitNode(content);
-            type->addContentType<Python::UnsureType>(contentVisitor.lastType());
-        }
-        foreach ( ExpressionAst* key, node->keys ) {
-            keyVisitor.visitNode(key);
-            type->addKeyType<Python::UnsureType>(keyVisitor.lastType());
+        Q_ASSERT(node->keys.length() == node->values.length());
+        for ( int ii = 0; ii < node->values.length(); ++ii ) {
+            contentVisitor.visitNode(node->values.at(ii));
+            if ( node->keys.at(ii) ) {
+                type->addContentType<Python::UnsureType>(contentVisitor.lastType());
+                keyVisitor.visitNode(node->keys.at(ii));
+                type->addKeyType<Python::UnsureType>(keyVisitor.lastType());
+            }
+            else if ( auto unpackedType = contentVisitor.lastType().cast<MapType>() ) {
+                // Key is null for `{**foo}`
+                type->addContentType<Python::UnsureType>(unpackedType->contentType().abstractType());
+                type->addKeyType<Python::UnsureType>(unpackedType->keyType().abstractType());
+            }
         }
     }
     encounter(AbstractType::Ptr::staticCast(type));
@@ -682,7 +654,7 @@ AbstractType::Ptr ExpressionVisitor::fromBinaryOperator(AbstractType::Ptr lhs, A
         if ( ! type ) {
             return AbstractType::Ptr();
         }
-        Declaration* func = Helper::accessAttribute(type->declaration(context()->topContext()), op, context());
+        auto func = Helper::accessAttribute(type, op, topContext());
         if ( ! func ) {
             return AbstractType::Ptr();
         }
