@@ -30,6 +30,7 @@
 #include <language/duchain/types/unsuretype.h>
 #include <language/duchain/types/integraltype.h>
 #include <language/duchain/types/containertypes.h>
+#include <language/duchain/types/functiontype.h>
 #include <language/duchain/duchainutils.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
@@ -48,6 +49,7 @@
 #include <shell/partcontroller.h>
 
 #include <KTextEditor/View>
+#include <KConfigGroup>
 
 #include "ast.h"
 #include "types/hintedtype.h"
@@ -60,9 +62,9 @@ using namespace KDevelop;
 
 namespace Python {
 
-QMap<IProject*, QList<QUrl>> Helper::cachedCustomIncludes;
-QList<QUrl> Helper::cachedSearchPaths;
-QList<QUrl> Helper::projectSearchPaths;
+QMap<IProject*, QVector<QUrl>> Helper::cachedCustomIncludes;
+QMap<IProject*, QVector<QUrl>> Helper::cachedSearchPaths;
+QVector<QUrl> Helper::projectSearchPaths;
 QStringList Helper::dataDirs;
 QString Helper::documentationFile;
 DUChainPointer<TopDUContext> Helper::documentationFileContext = DUChainPointer<TopDUContext>(0);
@@ -108,31 +110,35 @@ IndexedDeclaration Helper::declarationUnderCursor(bool allowUse)
     return KDevelop::IndexedDeclaration();
 }
 
-Declaration* Helper::accessAttribute(Declaration* accessed, const QString& attribute, const DUContext* current)
+Declaration* Helper::accessAttribute(const AbstractType::Ptr accessed,
+                                     const IndexedIdentifier& attribute,
+                                     const TopDUContext* topContext)
 {
-    if ( ! accessed || ! accessed->abstractType() ) {
+    if ( ! accessed ) {
         return 0;
     }
-    // if the type is unsure, search all the possibilities
-    auto structureTypes = Helper::filterType<StructureType>(accessed->abstractType(),
+    // if the type is unsure, search all the possibilities (but return the first match)
+    auto structureTypes = Helper::filterType<StructureType>(accessed,
         [](AbstractType::Ptr toFilter) {
             auto type = Helper::resolveAliasType(toFilter);
             return type && type->whichType() == AbstractType::TypeStructure;
+        },
+        [](AbstractType::Ptr toMap) {
+            return StructureType::Ptr::staticCast(Helper::resolveAliasType(toMap));
         }
     );
-    for ( auto type: structureTypes ) {
-        QList<DUContext*> searchContexts = Helper::internalContextsForClass(type, current->topContext());
-        for ( DUContext* c: searchContexts ) {
-            auto found = c->findDeclarations(KDevelop::Identifier(attribute),
-                                             CursorInRevision::invalid(),
-                                             current->topContext(), DUContext::DontSearchInParent);
-            std::reverse(found.begin(), found.end());
-            // never consider decls from the builtins
-            if ( ! found.isEmpty() && (
-                   found.first()->topContext() != Helper::getDocumentationFileContext() ||
-                   c->topContext() == Helper::getDocumentationFileContext() ) )
-            {
-                return found.first();
+    auto docFileContext = Helper::getDocumentationFileContext();
+
+    for ( const auto& type: structureTypes ) {
+        auto searchContexts = Helper::internalContextsForClass(type, topContext);
+        for ( const auto ctx: searchContexts ) {
+            auto found = ctx->findDeclarations(attribute, CursorInRevision::invalid(),
+                                               topContext, DUContext::DontSearchInParent);
+            if ( !found.isEmpty() && (
+                   found.last()->topContext() != docFileContext ||
+                   ctx->topContext() == docFileContext) ) {
+                // never consider decls from the builtins
+                return found.last();
             }
         }
     }
@@ -152,37 +158,22 @@ AbstractType::Ptr Helper::extractTypeHints(AbstractType::Ptr type)
     }));
 }
 
-Helper::FuncInfo Helper::functionDeclarationForCalledDeclaration(DeclarationPointer ptr)
+Helper::FuncInfo Helper::functionForCalled(Declaration* called, bool isAlias)
 {
-    if ( ! ptr ) {
-        return FuncInfo();
+    if ( ! called ) {
+        return { nullptr, false };
     }
-    bool isConstructor = false;
-    DeclarationPointer calledDeclaration = ptr;
-    if ( ! calledDeclaration->isFunctionDeclaration() ) {
-        // not a function -- try looking for a constructor
-        StructureType::Ptr classType = calledDeclaration->type<StructureType>();
-        auto contexts = Helper::internalContextsForClass(classType, ptr->topContext());
-        for ( DUContext* context: contexts ) {
-            static KDevelop::Identifier initIdentifier("__init__");
-            QList<Declaration*> constructors = context->findDeclarations(initIdentifier);
-            if ( ! constructors.isEmpty() ) {
-                calledDeclaration = dynamic_cast<FunctionDeclaration*>(constructors.first());
-                isConstructor = true;
-                break;
-            }
-        }
+    else if ( called->isFunctionDeclaration() ) {
+        return { static_cast<FunctionDeclaration*>( called ), false };
     }
-    FunctionDeclarationPointer lastFunctionDeclaration;
-    if ( calledDeclaration ) {
-        // It was a class -- use the constructor
-        lastFunctionDeclaration = calledDeclaration.dynamicCast<FunctionDeclaration>();
-    }
-    else {
-        // Use the original declaration
-        lastFunctionDeclaration = ptr.dynamicCast<FunctionDeclaration>();
-    }
-    return QPair<FunctionDeclarationPointer, bool>(lastFunctionDeclaration, isConstructor);
+    // If we're calling a type object (isAlias == true), look for a constructor.
+    static const IndexedIdentifier initId(KDevelop::Identifier("__init__"));
+
+    // Otherwise look for a `__call__()` method.
+    static const IndexedIdentifier callId(KDevelop::Identifier("__call__"));
+
+    auto attr = accessAttribute(called->abstractType(), (isAlias ? initId : callId), called->topContext());
+    return { dynamic_cast<FunctionDeclaration*>(attr), isAlias };
 }
 
 Declaration* Helper::declarationForName(const QualifiedIdentifier& identifier, const RangeInRevision& nodeRange,
@@ -227,23 +218,23 @@ Declaration* Helper::declarationForName(const QualifiedIdentifier& identifier, c
     return declaration;
 }
 
-QList< DUContext* > Helper::internalContextsForClass(StructureType::Ptr klassType, TopDUContext* context, ContextSearchFlags flags, int depth)
+QVector<DUContext*> Helper::internalContextsForClass(const StructureType::Ptr classType,
+                        const TopDUContext* context, ContextSearchFlags flags, int depth)
 {
-    QList<DUContext*> searchContexts;
-    if ( ! klassType ) {
+    QVector<DUContext*> searchContexts;
+    if ( ! classType ) {
         return searchContexts;
     }
-    if ( auto c = klassType->internalContext(context) ) {
+    if ( auto c = classType->internalContext(context) ) {
         searchContexts << c;
     }
-    Declaration* decl = Helper::resolveAliasDeclaration(klassType->declaration(context));
-    ClassDeclaration* klass = dynamic_cast<ClassDeclaration*>(decl);
-    if ( klass ) {
-        FOREACH_FUNCTION ( const BaseClassInstance& base, klass->baseClasses ) {
+    auto decl = Helper::resolveAliasDeclaration(classType->declaration(context));
+    if ( auto classDecl = dynamic_cast<ClassDeclaration*>(decl) ) {
+        FOREACH_FUNCTION ( const auto& base, classDecl->baseClasses ) {
             if ( flags == PublicOnly && base.access == KDevelop::Declaration::Private ) {
                 continue;
             }
-            StructureType::Ptr baseClassType = base.baseClass.type<StructureType>();
+            auto baseClassType = base.baseClass.type<StructureType>();
             // recursive call, because the base class will have more base classes eventually
             if ( depth < 10 ) {
                 searchContexts.append(Helper::internalContextsForClass(baseClassType, context, flags, depth + 1));
@@ -290,7 +281,6 @@ ReferencedTopDUContext Helper::getDocumentationFileContext()
         Helper::documentationFileContext = DUChainPointer<TopDUContext>(ctx.data());
         return ctx;
     }
-    return ReferencedTopDUContext(0); // c++...
 }
 
 // stolen from KUrl. duh.
@@ -371,13 +361,26 @@ QUrl Helper::getLocalCorrectionFile(const QUrl& document)
     return absolutePath;
 }
 
-QString getPythonExecutablePath() {
+QString Helper::getPythonExecutablePath(IProject* project)
+{
+    if ( project ) {
+        auto interpreter = project->projectConfiguration()->group("pythonsupport").readEntry("interpreter");
+        if ( !interpreter.isEmpty() ) {
+            // we have a user-configured interpreter, try using it
+            QFile f(interpreter);
+            if ( f.exists() ) {
+                return interpreter;
+            }
+            qCWarning(KDEV_PYTHON_DUCHAIN) << "Custom python interpreter" << interpreter << "configured for project" << project->name() << "is invalid, using default";
+        }
+    }
+
     // Find python 3 (https://www.python.org/dev/peps/pep-0394/)
-    auto result = QStandardPaths::findExecutable("python" PYTHON_VERSION_MAJOR "." PYTHON_VERSION_MINOR);
+    auto result = QStandardPaths::findExecutable("python" PYTHON_VERSION_STR);
     if ( ! result.isEmpty() ) {
         return result;
     }
-    result = QStandardPaths::findExecutable("python" PYTHON_VERSION_MAJOR);
+    result = QStandardPaths::findExecutable("python" PYTHON_VERSION_MAJOR_STR);
     if ( ! result.isEmpty() ) {
         return result;
     }
@@ -396,7 +399,7 @@ QString getPythonExecutablePath() {
         "HKEY_CURRENT_USER\\Software\\Python\\PythonCore\\PYTHON_VERSION\\InstallPath",
         "HKEY_CURRENT_USER\\Software\\Python\\PythonCore\\PYTHON_VERSION-32\\InstallPath"
     };
-    auto version = QString(PYTHON_VERSION_MAJOR) + "." + PYTHON_VERSION_MINOR;
+    auto version = QString(PYTHON_VERSION_STR);
     foreach ( QString key, keys ) {
         key.replace("PYTHON_VERSION", version);
         QSettings base(key.left(key.indexOf("Python")), QSettings::NativeFormat);
@@ -419,15 +422,15 @@ QString getPythonExecutablePath() {
     return PYTHON_EXECUTABLE;
 }
 
-QList<QUrl> Helper::getSearchPaths(const QUrl& workingOnDocument)
+QVector<QUrl> Helper::getSearchPaths(const QUrl& workingOnDocument)
 {
     QMutexLocker lock(&Helper::cacheMutex);
-    QList<QUrl> searchPaths;
+    QVector<QUrl> searchPaths;
     // search in the projects, as they're packages and likely to be installed or added to PYTHONPATH later
     // and also add custom include paths that are defined in the projects
 
+    auto project = ICore::self()->projectController()->findProjectForUrl(workingOnDocument);
     {
-        auto project = ICore::self()->projectController()->findProjectForUrl(workingOnDocument);
         QMutexLocker lock(&Helper::projectPathLock);
         searchPaths << Helper::projectSearchPaths;
         searchPaths << Helper::cachedCustomIncludes.value(project);
@@ -437,13 +440,14 @@ QList<QUrl> Helper::getSearchPaths(const QUrl& workingOnDocument)
         searchPaths.append(QUrl::fromLocalFile(path));
     }
 
-    if ( cachedSearchPaths.isEmpty() ) {
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "*** Gathering search paths...";
+    if ( !cachedSearchPaths.contains(project) ) {
+        QVector<QUrl> cachedForProject;
+        qCDebug(KDEV_PYTHON_DUCHAIN) << "*** Collecting search paths...";
         QStringList getpath;
         getpath << "-c" << "import sys; sys.stdout.write('$|$'.join(sys.path))";
         
         QProcess python;
-        python.start(getPythonExecutablePath(), getpath);
+        python.start(getPythonExecutablePath(project), getpath);
         python.waitForFinished(1000);
         QString pythonpath = QString::fromUtf8(python.readAllStandardOutput());
         auto paths = pythonpath.split("$|$");
@@ -451,7 +455,7 @@ QList<QUrl> Helper::getSearchPaths(const QUrl& workingOnDocument)
         
         if ( ! pythonpath.isEmpty() ) {
             foreach ( const QString& path, paths ) {
-                cachedSearchPaths.append(QUrl::fromLocalFile(path));
+                cachedForProject.append(QUrl::fromLocalFile(path));
             }
         }
         else {
@@ -461,16 +465,17 @@ QList<QUrl> Helper::getSearchPaths(const QUrl& workingOnDocument)
             QString path = qgetenv("PYTHONPATH");
             QStringList paths = path.split(':');
             foreach ( const QString& path, paths ) {
-                cachedSearchPaths.append(QUrl::fromLocalFile(path));
+                cachedForProject.append(QUrl::fromLocalFile(path));
             }
         }
         qCDebug(KDEV_PYTHON_DUCHAIN) << " *** Done. Got search paths: " << cachedSearchPaths;
+        cachedSearchPaths.insert(project, cachedForProject);
     }
     else {
         qCDebug(KDEV_PYTHON_DUCHAIN) << " --- Search paths from cache: " << cachedSearchPaths;
     }
     
-    searchPaths.append(cachedSearchPaths);
+    searchPaths.append(cachedSearchPaths.value(project));
     
     auto dir = workingOnDocument.adjusted(QUrl::RemoveFilename);
     if ( ! dir.isEmpty() ) {
@@ -486,31 +491,40 @@ bool Helper::isUsefulType(AbstractType::Ptr type)
     return TypeUtils::isUsefulType(type);
 }
 
-AbstractType::Ptr Helper::contentOfIterable(const AbstractType::Ptr iterable)
+AbstractType::Ptr Helper::contentOfIterable(const AbstractType::Ptr iterable, const TopDUContext* topContext)
 {
-    auto items = filterType<AbstractType>(iterable,
-        [](AbstractType::Ptr t) {
-            return ListType::Ptr::dynamicCast(t) || IndexedContainer::Ptr::dynamicCast(t);
-        },
-        [](AbstractType::Ptr t) {
-            if ( auto variable = ListType::Ptr::dynamicCast(t) ) {
-                return AbstractType::Ptr(variable->contentType().abstractType());
-            }
-            else {
-                auto indexed = t.cast<IndexedContainer>();
-                return indexed->asUnsureType();
+    auto types = filterType<StructureType>(iterable,
+        [](AbstractType::Ptr t) { return t->whichType() == AbstractType::TypeStructure; } );
+
+    static const IndexedIdentifier iterId(KDevelop::Identifier("__iter__"));
+    static const IndexedIdentifier nextId(KDevelop::Identifier("__next__"));
+    AbstractType::Ptr content(new IntegralType(IntegralType::TypeMixed));
+
+    for ( const auto& type: types ) {
+        if ( auto map = type.cast<MapType>() ) {
+            // Iterating over dicts gets keys, not values
+            content = mergeTypes(content, map->keyType().abstractType());
+            continue;
+        }
+        else if ( auto list = type.cast<ListType>() ) {
+            content = mergeTypes(content, list->contentType().abstractType());
+            continue;
+        }
+        else if ( auto indexed = type.cast<IndexedContainer>() ) {
+            content = mergeTypes(content, indexed->asUnsureType());
+            continue;
+        }
+        DUChainReadLocker lock;
+        // Content of an iterable object is iterable.__iter__().__next__().
+        if ( auto iterFunc = dynamic_cast<FunctionDeclaration*>(accessAttribute(type, iterId, topContext)) ) {
+            if ( auto iterator = iterFunc->type<FunctionType>()->returnType().cast<StructureType>() ) {
+                if ( auto nextFunc = dynamic_cast<FunctionDeclaration*>(accessAttribute(iterator, nextId, topContext)) ) {
+                    content = mergeTypes(content, nextFunc->type<FunctionType>()->returnType());
+                }
             }
         }
-    );
-
-    if ( items.size() == 1 ) {
-        return items.first();
     }
-    auto unsure = AbstractType::Ptr(new UnsureType);
-    for ( auto type: items ) {
-        Helper::mergeTypes(unsure, type);
-    }
-    return unsure;
+    return content;
 }
 
 AbstractType::Ptr Helper::mergeTypes(AbstractType::Ptr type, const AbstractType::Ptr newType)
