@@ -50,7 +50,7 @@ namespace Python {
 class KDEVPYTHONDUCHAIN_EXPORT Helper {
 public:
     /** get search paths for python files **/
-    static QList<QUrl> getSearchPaths(const QUrl& workingOnDocument);
+    static QVector<QUrl> getSearchPaths(const QUrl& workingOnDocument);
     static QStringList dataDirs;
     static QString documentationFile;
     static QStringList correctionFileDirs;
@@ -65,15 +65,31 @@ public:
     static QUrl getLocalCorrectionFile(const QUrl& document);
 
     static QMutex cacheMutex;
-    static QList<QUrl> cachedCustomIncludes;
-    static QList<QUrl> cachedSearchPaths;
+    static QMap<IProject*, QVector<QUrl>> cachedCustomIncludes;
+    static QMap<IProject*, QVector<QUrl>> cachedSearchPaths;
 
     static QMutex projectPathLock;
-    static QList<QUrl> projectSearchPaths;
+    static QVector<QUrl> projectSearchPaths;
 
-    static AbstractType::Ptr extractTypeHints(AbstractType::Ptr type, TopDUContext* current);
+    static AbstractType::Ptr extractTypeHints(AbstractType::Ptr type);
 
-    static Declaration* accessAttribute(Declaration* accessed, const QString& attribute, const DUContext* current);
+    /**
+     * @brief Get the declaration of 'accessed.attribute', or return null.
+     *
+     * @param accessed Type (Structure or Unsure) that should have this attribute.
+     * @param attribute Which attribute to look for.
+     * @param topContext Top context (for this file?)
+     * @return Declaration* of the attribute, or null.
+     *  If UnsureType with >1 matching attributes, returns an arbitrary choice.
+     **/
+    static KDevelop::Declaration* accessAttribute(const KDevelop::AbstractType::Ptr accessed,
+                                                  const KDevelop::IndexedIdentifier& attribute,
+                                                  const KDevelop::TopDUContext* topContext);
+
+    static KDevelop::Declaration* accessAttribute(const KDevelop::AbstractType::Ptr accessed,
+        const QString& attribute, const KDevelop::TopDUContext* topContext) {
+        return accessAttribute(accessed, IndexedIdentifier(KDevelop::Identifier(attribute)), topContext);
+    }
 
     static AbstractType::Ptr resolveAliasType(const AbstractType::Ptr eventualAlias);
 
@@ -83,7 +99,7 @@ public:
      * @param iterable Type to get the contents of. Can be an unsure.
      * @return KDevelop::AbstractType::Ptr Content type. Might be an unsure.
      */
-    static AbstractType::Ptr contentOfIterable(const AbstractType::Ptr iterable);
+    static AbstractType::Ptr contentOfIterable(const AbstractType::Ptr iterable, const TopDUContext* topContext);
 
     /**
      * @brief Get a list of types inside the passed type which match the specified filter.
@@ -121,16 +137,17 @@ public:
 
     static KDevelop::IndexedDeclaration declarationUnderCursor(bool allowUse = true);
 
+    struct FuncInfo { FunctionDeclaration* declaration; bool isConstructor; };
     /**
      * @brief Finds whether the specified called declaration is a function declaration, and if not,
      *        checks for a class declaration; then returns the constructor
      *
-     * @param ptr the declaration to check
+     * @param called the declaration to check
+     * @param isAlias whether the called declaration aliases a class or function definition.
      * @return the function pointer which was found, or an invalid pointer, and a bool
      *         which is true when it is a constructor
      **/
-    using FuncInfo = QPair<Python::FunctionDeclarationPointer, bool>;
-    static FuncInfo functionDeclarationForCalledDeclaration(DeclarationPointer ptr);
+    static FuncInfo functionForCalled(Declaration* called, bool isAlias=true);
 
     template<typename T> static const Decorator* findDecoratorByName(T* inDeclaration, const QString& name) {
         const int count = inDeclaration->decoratorsSize();
@@ -142,9 +159,8 @@ public:
         return 0;
     };
 
-    static bool docstringContainsHint(Declaration* declaration, const QString& hintName, QStringList* args = 0) {
+    static bool docstringContainsHint(const QString& comment, const QString& hintName, QStringList* args = 0) {
         // TODO cache types! this is horribly inefficient
-        const QString& comment = declaration->comment();
         const QString search = "! " + hintName + " !";
         int index = comment.indexOf(search);
         if ( index >= 0 ) {
@@ -191,12 +207,13 @@ public:
     /**
     * @brief Find all internal contexts for this class and its base classes recursively
     *
-    * @param klass Type object for the class to search contexts
+    * @param classType Type object for the class to search contexts
     * @param context TopContext for finding the declarations for types
     * @return list of contexts which were found
     **/
-    static QList<DUContext*> internalContextsForClass(KDevelop::StructureType::Ptr klassType,
-                                                      TopDUContext* context, ContextSearchFlags flags = NoFlags, int depth = 0);
+    static QVector<DUContext*> internalContextsForClass(const KDevelop::StructureType::Ptr classType,
+                                                        const TopDUContext* context,
+                                                        ContextSearchFlags flags = NoFlags, int depth = 0);
     /**
       * @brief Resolve the given declaration if it is an alias declaration.
       *
@@ -208,6 +225,8 @@ public:
 
     static Declaration* declarationForName(const QualifiedIdentifier& identifier, const RangeInRevision& nodeRange,
                                            DUChainPointer<const DUContext> context);
+
+    static QString getPythonExecutablePath(IProject* project);
 };
 
 }
