@@ -418,6 +418,13 @@ void PyDUChainTest::testCrashes_data() {
 
     QTest::newRow("list_extend_missing") << "foo = []\nfoo.extend(missing)";
     QTest::newRow("list_extend_missing_arg") << "foo = []\nfoo.extend()";
+    QTest::newRow("method_of_call_with_list_arg") <<
+        "class MyClass:\n"
+        "    def bar(self): pass\n"
+        "def foo(x):\n"
+        "    return MyClass()\n"
+        "foo([0]).bar()";
+    QTest::newRow("unpacked_dict_kwarg") << "def foo(arg): pass\nfoo(**{'arg': 2})";
 }
 
 void PyDUChainTest::testClassVariables()
@@ -704,7 +711,7 @@ void PyDUChainTest::testRanges()
         visitor->searchingForRange = r;
         visitor->searchingForIdentifier = identifier;
         visitor->visitCode(m_ast.data());
-
+        QEXPECT_FAIL("attr_dot_name_hash", "Insufficiently magic hack", Continue);
         QCOMPARE(visitor->found, true);
         delete visitor;
     }
@@ -733,6 +740,11 @@ void PyDUChainTest::testRanges_data()
     QTest::newRow("attr_of_string_in_list") << "[\"*{0}*\".format(foo)]" << 1 << ( QStringList() << "9,14,format" );
     QTest::newRow("attr_of_call_in_list") << "[foo().format(foo)]" << 1 << ( QStringList() << "7,12,format" );
     QTest::newRow("attr_parentheses") << "(\"foo\" + \"foo\").capitalize()" << 1 << ( QStringList() << "16,25,capitalize" );
+    QTest::newRow("attr_commented_name") << "base.attr # attr" << 2 << ( QStringList() << "5,8,attr" );
+    QTest::newRow("attr_name_in_strings") << "'attr' + base['attr'].attr # attr" << 4 << ( QStringList() << "22,25,attr" );
+    QTest::newRow("attr_dot_hash_in_strings") << "'.foo#' + base['.#'].attr # attr" << 4 << ( QStringList() << "21,24,attr" );
+    QTest::newRow("attr_dot_name_hash") << "base['.attr#'].attr" << 4 << ( QStringList() << "15,18,attr" );
+
     QTest::newRow("string_parentheses") << "(\"asdf\".join())" << 1 << ( QStringList() << "8,11,join" );
     QTest::newRow("string_parentheses2") << "(\"asdf\").join()" << 1 << ( QStringList() << "9,12,join" );
     QTest::newRow("string_parentheses3") << "(\"asdf\".join()).join()" << 2 << ( QStringList() << "8,11,join" << "16,19,join" );
@@ -788,8 +800,9 @@ void PyDUChainTest::testTypes()
     visitor->ctx = TopDUContextPointer(ctx.data());
     visitor->searchingForType = expectedType;
     visitor->visitCode(m_ast.data());
-    QEXPECT_FAIL("lambda", "not implemented: aliasing lambdas", Continue);
-    QEXPECT_FAIL("tuple_unsure", "not implemented, rework DeclarationBuilder::sourcesOfAssignment", Continue);
+    QEXPECT_FAIL("return_builtin_iterator", "fake builtin iter()", Continue);
+    QEXPECT_FAIL("parent_constructor_arg_type", "Not enough passes?", Continue);
+    QEXPECT_FAIL("init_class_no_decl", "aliasing info lost", Continue);
     QCOMPARE(visitor->found, true);
 }
 
@@ -822,6 +835,11 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("arg_after_vararg") << "def func(x, y, *, z:int): return z\ncheckme = func()" << "int";
     QTest::newRow("arg_after_vararg_with_default") << "def func(x=5, y=3, *, z:int): return z\ncheckme = func()" << "int";
 
+    QTest::newRow("class_scope_end_inside") << "a = str()\nclass M:\n"
+                                               "  a = 2\n  foo = a\n"
+                                               "checkme = M().foo" << "int";
+    QTest::newRow("class_scope_end_outside") << "a = str()\nclass M:\n  a = 2\ncheckme = a" << "str";
+
     QTest::newRow("list_access_right_open_slice") << "some_list = []; checkme = some_list[2:]" << "list";
     QTest::newRow("list_access_left_open_slice") << "some_list = []; checkme = some_list[:2]" << "list";
     QTest::newRow("list_access_closed_slice") << "some_list = []; checkme = some_list[2:17]" << "list";
@@ -837,6 +855,7 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("tuple2") << "foo, checkme = 3, \"str\"" << "str";
     QTest::newRow("tuple2_negative_index") << "foo = (1, 2, 'foo')\ncheckme = foo[-1]" << "str";
     QTest::newRow("tuple_type") << "checkme = 1, 2" << "tuple";
+    QTest::newRow("tuple_rhs_unpack") << "foo = 1, 2.5\nbar = 1, *foo, 2\ncheckme = bar[2]" << "float";
     
     QTest::newRow("dict_iteritems") << "d = {1:2, 3:4}\nfor checkme, k in d.iteritems(): pass" << "int";
     QTest::newRow("enumerate_key") << "d = [str(), str()]\nfor checkme, value in enumerate(d): pass" << "int";
@@ -881,7 +900,6 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("tuple_nested2") << "mytuple = 3, ('foo', 5.5)\nfoobar, (checkme, other) = mytuple" << "str";
     QTest::newRow("tuple_nested3") << "mytuple = ((7, 'foo'), 5.5), 3\n((baz, checkme), other), foo = mytuple" << "str";
 
-    // This isn't actually defined behaviour, but it works in CPython, so people use it...
     QTest::newRow("tuple_nested_ext") << "mytuple = (2, ('foo', 'bar', 6), 7)\na, (b, *checkme, c), *d = mytuple" << "list of str";
 
     QTest::newRow("tuple_multi_assign") << "mytuple = 2, 'foo'\ncheckme = a = mytuple" << "tuple";
@@ -925,15 +943,114 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("no_hints_type") << "def myfun(arg): arg = 3; return arg\ncheckme = myfun(3)" << "int";
     QTest::newRow("hints_type") << "def myfun(arg): return arg\ncheckme = myfun(3)" << "int";
     QTest::newRow("args_type") << "def myfun(*args): return args[0]\ncheckme = myfun(3)" << "int";
-    QTest::newRow("kwarg_type") << "def myfun(**args): return args[0]\ncheckme = myfun(a=3)" << "int";
+    QTest::newRow("kwarg_type") << "def myfun(**kwargs): return kwargs['a']\ncheckme = myfun(a=3)" << "int";
+    QTest::newRow("dict_kwarg_type") << "def foo(**kwargs): return kwargs['']\ncheckme = foo(**{'a': 12})" << "int";
+    QTest::newRow("dict_norm_kwarg_type") << "def foo(**kwargs): return kwargs['']\n"
+                                             "checkme = foo(**{'a': 12}, b=1.2)" << "unsure (int, float)";
+    QTest::newRow("multi_dict_kwarg_type") << "def foo(**kwargs): return kwargs['']\n"
+                                              "checkme = foo(**{'a': 12}, b=1.2, **{'c': ''})" << "unsure (int, float, str)";
+    QTest::newRow("named_arg_type") << "def myfun(arg): return arg\ncheckme = myfun(arg=3)" << "int";
 
-    QTest::newRow("tuple_unsure") << "q = (3, str())\nq=(str(), 3)\ncheckme, _ = q" << "unsure(int, str)";
+    QTest::newRow("arg_args_type") << "def myfun(arg, *args): return args[0]\n"
+                                      "checkme = myfun(3, str())" << "str";
+    QTest::newRow("arg_kwargs_type") << "def myfun(arg, **kwargs): return kwargs['a']\n"
+                                         "checkme = myfun(12, a=str())" << "str";
+    QTest::newRow("named_kwargs_type_1") << "def myfun(arg, **kwargs): return arg\n"
+                                            "checkme = myfun(arg=12, a=str())" << "int";
+    QTest::newRow("named_kwargs_type_2") << "def myfun(arg, **kwargs): return kwargs['a']\n"
+                                            "checkme = myfun(arg=12, a=str())" << "str";
+    QTest::newRow("kwargs_named_type") << "def myfun(arg, **kwargs): return kwargs['a']\n"
+                                          "checkme = myfun(a=str(), arg=12)" << "str";
+    QTest::newRow("varied_args_type_1") << "def myfun(arg, *args, **kwargs): return arg\n"
+                                           "checkme = myfun(1, 1.5, a=str())" << "int";
+    QTest::newRow("varied_args_type_2") << "def myfun(arg, *args, **kwargs): return args[0]\n"
+                                           "checkme = myfun(1, 1.5, a=str())" << "float";
+    QTest::newRow("varied_args_type_3") << "def myfun(arg, *args, **kwargs): return kwargs['a']\n"
+                                           "checkme = myfun(1, 1.5, a=str())" << "str";
+    QTest::newRow("method_args_type_1") << "class MyClass:\n"
+                                           "   def method(self, arg): return self\n"
+                                           "checkme = MyClass().method(12)" << "MyClass";
+    QTest::newRow("method_args_type_2") << "class MyClass:\n"
+                                           "   def method(self, arg): return arg\n"
+                                           "checkme = MyClass().method(12)" << "int";
+    QTest::newRow("clsmethod_args_type_1") << "class MyClass:\n"
+                                              "   @classmethod\n"
+                                              "   def method(cls, arg): return cls\n"
+                                              "checkme = MyClass().method(12)" << "MyClass";
+    QTest::newRow("clsmethod_args_type_2") << "class MyClass:\n"
+                                              "   @classmethod\n"
+                                              "   def method(cls, arg): return arg\n"
+                                              "checkme = MyClass().method(12)" << "int";
+    QTest::newRow("staticmethod_args_type") << "class MyClass:\n"
+                                               "   @staticmethod\n"
+                                               "   def method(arg): return arg\n"
+                                               "checkme = MyClass().method(12)" << "int";
+    QTest::newRow("staticmethod_vararg_type") << "class MyClass:\n"
+                                                 "   @staticmethod\n"
+                                                 "   def method(arg, *args): return args[0]\n"
+                                                 "checkme = MyClass().method(12, 2.5)" << "float";
+    QTest::newRow("method_explicit_self") << "class MyClass:\n"
+                                             "   def method(self, arg): return arg\n"
+                                             "instance = MyClass()\n"
+                                             "checkme = MyClass.method(instance, 12)" << "int";
+    QTest::newRow("method_vararg_explicit_self") << "class MyClass:\n"
+                                                    "    def foo(self, arg, *args): return args[0]\n"
+                                                    "mc = MyClass()\n"
+                                                    "checkme = MyClass.foo(mc, 'str', 3, 4.5)" << "int";
+    QTest::newRow("clsmethod_explicit_self") << "class MyClass:\n"
+                                                "   @classmethod\n"
+                                                "   def method(cls, arg1, arg2): return arg2\n"
+                                                "instance = MyClass()\n"
+                                                "checkme = MyClass.method('a', 12)" << "int";
+    QTest::newRow("staticmethod_explicit_self") << "class MyClass:\n"
+                                                   "   @staticmethod\n"
+                                                   "   def method(arg1, arg2): return arg1\n"
+                                                   "instance = MyClass()\n"
+                                                   "checkme = MyClass.method('a', 12)" << "str";
+    QTest::newRow("parent_constructor_arg_type") << "class Base:\n" // https://bugs.kde.org/show_bug.cgi?id=369364
+                                                    "    def __init__(self, foo):\n"
+                                                    "        self.foo = foo\n"
+                                                    "class Derived(Base):\n"
+                                                    "    def __init__(self, foo):\n"
+                                                    "       Base.__init__(self, foo)\n"
+                                                    "instance = Derived('string')\n"
+                                                    "checkme = instance.foo" << "str";
 
+    QTest::newRow("tuple_unsure") << "q = (3, str())\nq=(str(), 3)\ncheckme, _ = q" << "unsure (int, str)";
+
+    QTest::newRow("custom_iterable") << "class Gen2:\n"
+                                        "    def __iter__(self): return self\n"
+                                        "    def __next__(self): return 'blah'\n"
+                                        "for checkme in Gen2(): pass" << "str";
+    QTest::newRow("separate_iterator") << "class Foo:\n"
+                                          "    def __iter__(self): return Bar()\n"
+                                          "    def __next__(self): return 'blah'\n" // Not used (or shouldn't be!)
+                                          "class Bar:\n"
+                                          "    def __next__(self): return {1}\n"
+                                          "checkme = [a for a in Foo()]" << "list of set of int";
+    QTest::newRow("return_builtin_iterator") << "class Gen2:\n"
+                                                "    contents = [1, 2, 3]\n"
+                                                "    def __iter__(self): return iter(Gen2.contents)\n"
+                                                "for checkme in Gen2(): pass" << "int";
+
+    QTest::newRow("init_class") <<  "class Foo:\n"
+                                    "  def __init__(self): pass\n"
+                                    "  def __call__(self): return 1.5\n"
+                                    "checkme = Foo()\n" << "Foo";
+    QTest::newRow("init_class_no_decl") <<  "class Foo:\n"
+                                            "  def __init__(self): pass\n"
+                                            "  def __call__(self): return 1.5\n"
+                                            "a = [Foo]\n"
+                                            "checkme = a[0]()\n" << "Foo";
     QTest::newRow("call_class") << "class Foo:\n"
                                     "    def __call__(self):\n"
                                     "         return 0\n"
                                     "f = Foo()\n"
                                     "checkme = f()\n" << "int";
+    QTest::newRow("call_class_no_decl") << "class Foo:\n"
+                                           "  def __call__(self): return 1.5\n"
+                                           "a = [Foo()]\n"
+                                           "checkme = a[0]()" << "float";
     QTest::newRow("classmethod") << "class Foo:\n"
                                     "    @classmethod\n"
                                     "    def foo(cls):\n"
@@ -1357,15 +1474,20 @@ void PyDUChainTest::testContainerTypes_data()
     QTest::addColumn<QString>("code");
     QTest::addColumn<QString>("contenttype");
     QTest::addColumn<bool>("use_type");
-    
+
     QTest::newRow("list_of_int") << "checkme = [1, 2, 3]" << "int" << false;
+    QTest::newRow("list_from_unpacked") << "foo = [1.3]\ncheckme = [1, *foo, 3]" << "unsure (int, float)" << false;
     QTest::newRow("list_of_int_call") << "checkme = list([1, 2, 3])" << "int" << false;
     QTest::newRow("generator") << "checkme = [i for i in [1, 2, 3]]" << "int" << false;
     QTest::newRow("list_access") << "list = [1, 2, 3]\ncheckme = list[0]" << "int" << true;
     QTest::newRow("set_of_int") << "checkme = {1, 2, 3}" << "int" << false;
+    QTest::newRow("set_from_unpacked") << "foo = [1.3]\ncheckme = {1, *foo, 3}" << "unsure (int, float)" << false;
     QTest::newRow("set_of_int_call") << "checkme = set({1, 2, 3})" << "int" << false;
     QTest::newRow("set_generator") << "checkme = {i for i in [1, 2, 3]}" << "int" << false;
     QTest::newRow("dict_of_int") << "checkme = {a:1, b:2, c:3}" << "int" << false;
+    QTest::newRow("dict_from_unpacked") << "checkme = {**{'a': 1}}" << "dict of str : int" << true;
+    QTest::newRow("dict_from_varied") << "checkme = {**{'a': 1}, 1: 1.5}" <<
+                                         "dict of unsure (str, int) : unsure (int, float)" << true;
     QTest::newRow("dict_of_int_call") << "checkme = dict({a:1, b:2, c:3})" << "int" << false;
     QTest::newRow("dict_generator") << "checkme = {\"Foo\":i for i in [1, 2, 3]}" << "int" << false;
     QTest::newRow("dict_access") << "list = {a:1, b:2, c:3}\ncheckme = list[0]" << "int" << true;
@@ -1382,6 +1504,26 @@ void PyDUChainTest::testContainerTypes_data()
     QTest::newRow("for_loop_tuple_2") << "d = [(3, 3.5)]\nfor a, b in d:\n checkme = b" << "float" << true;
     QTest::newRow("for_loop_tuple_unsure") << "d = [(3, 3.5), (3.5, 3)]\nfor a, b in d:\n checkme = b"
                                            << "unsure (float, int)" << true;
+    // Proposed by Nicolás Alvarez; why not? https://bugs.kde.org/show_bug.cgi?id=359915
+    QTest::newRow("comprehension_messy") << "users = {'a':19, 'b':42, 'c':35}\n"
+                                            "sorted_list = sorted(users.items(), key=lambda kv: (-kv[1], kv[0]))\n"
+                                            "checkme = [k for r,(k,v) in enumerate(sorted_list, 1)]" << "list of str" << true;
+    QTest::newRow("comprehension_multiline") << "checkme = [a for\n a in \n (1, 2)]" << "list of int" << true;
+    // From https://bugs.kde.org/show_bug.cgi?id=359912
+    QTest::newRow("subscript_multi") <<
+        "class Middle:\n def __getitem__(self, key):\n  return str()\n"
+        "class Outer:\n def __getitem__(self, key):\n  return Middle()\n"
+        "aaa = Outer()\ncheckme = aaa[0][0]" << "str" << true;
+    QTest::newRow("subscript_func_call") <<
+        "class Foo:\n def __getitem__(self, key):\n  return str()\n"
+        "def bar():\n return Foo()\n"
+        "checkme = bar()[0]" << "str" << true;
+    QTest::newRow("subscript_unknown_index") << "a = 1,str()\ncheckme = a[5-4]" << "unsure (int, str)" << true;
+    QTest::newRow("subscript_unsure") << "a = 1,2\na=[str()]\ncheckme = a[0]" << "unsure (int, str)" << true;
+    QTest::newRow("subscript_unsure_getitem") <<
+        "class Foo:\n def __getitem__(self, key):\n  return str()\n"
+        "class Bar:\n def __getitem__(self, key):\n  return float()\n"
+        "a = Foo()\na=Bar()\na=[1,2]\ncheckme = a[1]" << "unsure (str, float, int)" << true;
 }
 
 void PyDUChainTest::testVariableCreation()
@@ -1427,11 +1569,18 @@ void PyDUChainTest::testVariableCreation_data()
     QTest::newRow("unpack_from_list_inplace") << "a, b = [1, 2, 3]" << QStringList{"a", "b"} << QStringList{"int", "int"};
     QTest::newRow("unpack_from_list_indirect") << "c = [1, 2, 3]\na, b = c" << QStringList{"a", "b"}
                                                                             << QStringList{"int", "int"};
+    QTest::newRow("unpack_custom_iterable") <<
+        "class Foo:\n"
+        "    def __iter__(self): return self\n"
+        "    def __next__(self): return 1.5\n"
+        "a, *b = Foo()" << QStringList{"a", "b"} << QStringList {"float", "list of float"};
     QTest::newRow("for_loop_simple") << "for i in range(3): pass" << QStringList{"i"} << QStringList{"int"};
     QTest::newRow("for_loop_unpack") << "for a, b in [(3, 5.1)]: pass" << QStringList{"a", "b"}
                                                                        << QStringList{"int", "float"};
-    QTest::newRow("for_loop_stacked") << "for a, (b, c) in [(1, 2, 3.5)]: pass" << QStringList{"a", "b", "c"}
+    QTest::newRow("for_loop_stacked") << "for a, (b, c) in [(1, (2, 3.5))]: pass" << QStringList{"a", "b", "c"}
                                                                                 << QStringList{"int", "int", "float"};
+    QTest::newRow("for_loop_tuple") << "for a in 1, 2: pass" << QStringList{"a"} << QStringList{"int"};
+    QTest::newRow("for_loop_dict") << "for a in {'foo': 1}: pass" << QStringList{"a"} << QStringList{"str"};
 }
 
 void PyDUChainTest::testCleanupMultiplePasses()
