@@ -93,6 +93,7 @@ public:
         ContinueAstType,
         AssertionAstType,
         AugmentedAssignmentAstType,
+        AnnotationAssignmentAstType,
         LastStatementType,
         ExpressionAstType, // everything below is an expression
         AwaitAstType,
@@ -116,6 +117,8 @@ public:
         CompareAstType,
         NumberAstType,
         StringAstType,
+        JoinedStringAstType,
+        FormattedValueAstType,
         BytesAstType,
         SubscriptAstType,
         StarredAstType,
@@ -325,6 +328,14 @@ public:
     ExpressionAst* value;
 };
 
+class KDEVPYTHONPARSER_EXPORT AnnotationAssignmentAst : public StatementAst {
+public:
+    AnnotationAssignmentAst(Ast* parent);
+    ExpressionAst* target;
+    ExpressionAst* value;
+    ExpressionAst* annotation;
+};
+
 class KDEVPYTHONPARSER_EXPORT ForAst : public StatementAst {
 public:
     ForAst(Ast* parent);
@@ -443,7 +454,6 @@ public:
         Invalid = -1
     };
     ExpressionAst* value; // WARNING this is not set in most cases!
-    CallAst* belongsToCall;
 };
 
 class KDEVPYTHONPARSER_EXPORT AwaitAst : public ExpressionAst {
@@ -526,7 +536,7 @@ public:
 class KDEVPYTHONPARSER_EXPORT DictAst : public ExpressionAst {
 public:
     DictAst(Ast* parent);
-    QList<ExpressionAst*> keys;
+    QList<ExpressionAst*> keys; // WARNING: Can contain null elements: `{**other}`
     QList<ExpressionAst*> values;
 };
 
@@ -594,6 +604,20 @@ public:
     bool usedAsComment;
 };
 
+class KDEVPYTHONPARSER_EXPORT JoinedStringAst : public ExpressionAst {
+public:
+    JoinedStringAst(Ast* parent);
+    QList<ExpressionAst*> values;
+};
+
+class KDEVPYTHONPARSER_EXPORT FormattedValueAst : public ExpressionAst {
+public:
+    FormattedValueAst(Ast* parent);
+    ExpressionAst* value;
+    int conversion;
+    ExpressionAst* formatSpec;
+};
+
 class KDEVPYTHONPARSER_EXPORT BytesAst : public ExpressionAst {
 public:
     BytesAst(Ast* parent);
