@@ -29,6 +29,8 @@
 #include <language/duchain/problem.h>
 #include <language/duchain/duchain.h>
 
+#include <memory>
+
 #include "python_header.h"
 #include "astdefaultvisitor.h"
 #include "cythonsyntaxremover.h"
@@ -50,7 +52,7 @@ namespace Python
 // Update the "end" cursors of all nodes in the given tree.
 class RangeUpdateVisitor : public AstDefaultVisitor {
 public:
-    virtual void visitNode(Ast* node) {
+    void visitNode(Ast* node) override {
         AstDefaultVisitor::visitNode(node);
         if ( node && node->parent && node->parent->astType != Ast::AttributeAstType ) {
             if ( ( node->parent->endLine <= node->endLine && node->parent->endCol <= node->endCol )
@@ -68,7 +70,7 @@ public:
     KTextEditor::Cursor findNext(Python::Ast* node) {
         m_root = node;
         auto parent = node;
-        while ( parent && parent->parent && parent->parent->isExpression() ) {
+        while ( parent->parent && parent->parent->isExpression() ) {
             parent = parent->parent;
         }
         visitNode(parent);
@@ -81,7 +83,7 @@ public:
 
         return m_next;
     };
-    virtual void visitNode(Python::Ast* node) override {
+    void visitNode(Python::Ast* node) override {
         if ( ! node ) {
             return;
         }
@@ -112,16 +114,16 @@ class RangeFixVisitor : public AstDefaultVisitor {
 public:
     RangeFixVisitor(const QString& contents)
         : lines(contents.split('\n')) { };
-    virtual void visitFunctionDefinition(FunctionDefinitionAst* node) override {
+    void visitFunctionDefinition(FunctionDefinitionAst* node) override {
         cutDefinitionPreamble(node->name, node->async ? "asyncdef" : "def");
         AstDefaultVisitor::visitFunctionDefinition(node);
     };
-    virtual void visitClassDefinition(ClassDefinitionAst* node) override {
+    void visitClassDefinition(ClassDefinitionAst* node) override {
         cutDefinitionPreamble(node->name, "class");
         AstDefaultVisitor::visitClassDefinition(node);
     };
 
-    virtual void visitAttribute(AttributeAst* node) override {
+    void visitAttribute(AttributeAst* node) override {
         // Work around the weird way to count columns in Python's AST module.
 
         // Find where the next expression (of any kind) behind this one starts
@@ -133,30 +135,66 @@ public:
         }
 
         // take only the portion of the line up to that next expression
-        QString line = lines.at(node->startLine);
-        auto lineno = next_start.line();
-        auto colno = next_start.column();
+        auto endLine = next_start.line();
+        auto endCol = next_start.column();
         if ( ! (next_start > node->start()) ) {
-            colno = -1;
-            lineno = node->startLine;
+            endLine = node->startLine;
+            endCol = -1;
         }
-        do {
-            line = lines.at(lineno);
-            if ( colno != -1 ) {
-                line = line.left(colno);
-            }
-            colno = -1;
-            lineno -= 1;
-            // eventually go to previous line for multi-line expressions
-        } while ( ! line.contains(node->attribute->value) && lineno >= 0 );
 
-        node->startLine = lineno + 1;
+        const QString& name(node->attribute->value);
+
+        QString line;
+        for ( int n = node->startLine,
+                  pos = node->value->endCol + 1,
+                  dotFound = false,
+                  nameFound = false;
+               n <= endLine; ++n, pos = 0 ) {
+            line = lines.at(n);
+            if ( n == endLine && endCol != -1 ) {
+                // Never look at the next expression.
+                line = line.left(endCol);
+            }
+            if ( !dotFound ) {
+                // The real attr name can never be before a dot.
+                // Nor can the start of a comment.
+                // (Don't be misled by `foo["bar"].bar` or `foo["#"].bar`)
+                pos = line.indexOf('.', pos);
+                if ( pos == -1 ) continue;
+                dotFound = true;
+            }
+            if ( !nameFound ) {
+                // Track if the attr name has appeared at least once.
+                // This helps avoid interpreting '#'s in strings as comments -
+                //   there can never be a comment before the real attr name.
+                pos = line.indexOf(name, pos + 1);
+                if ( pos == -1 ) continue;
+                nameFound = true;
+            }
+            if ( dotFound && nameFound &&
+                  (pos = line.indexOf('#', pos + name.length())) != -1) {
+                // Remove the comment after a '#' iff we're certain it can't
+                //  be inside a string literal (e.g. `foo["#"].bar`).
+                line = line.left(pos);
+            }
+            // Take the last occurrence, any others are in string literals.
+            pos = line.lastIndexOf(name);
+            if ( pos != -1 ) {
+                node->startLine = n;
+                node->startCol = pos;
+            }
+            // N.B. we do this for all lines, the last non-comment occurrence
+            //  is the real one.
+        }
+        // This fails (only, AFAIK) in a very limited case:
+        // If the value expression (`foo` in `foo.bar`) contains a dot, the
+        //   attr name, _and_ a hash in that order (may not be consecutive),
+        //   and the hash is on the same line as the real attr name,
+        //   we wrongly interpret the hash as the start of a comment.
+        // e.g `foo["...barrier#"].bar` will highlight part of the string.
+
         node->endLine = node->startLine;
-        // now, just take the last occurence of the value.
-        // it is guaranteed that this is the right one, otherwise
-        // that portion of the line would have been cut off above.
-        node->startCol = line.lastIndexOf(node->attribute->value);
-        node->endCol = node->startCol + node->attribute->value.length() - 1;
+        node->endCol = node->startCol + name.length() - 1;
         node->attribute->copyRange(node);
 
         AstDefaultVisitor::visitAttribute(node);
@@ -165,7 +203,7 @@ public:
     // alias for imports (import foo as bar, baz as bang)
     // no strings, brackets, or whatever are allowed here, so the "parser"
     // can be very straightforward.
-    virtual void visitImport(ImportAst* node) override {
+    void visitImport(ImportAst* node) override {
         AstDefaultVisitor::visitImport(node);
         int aliasIndex = 0;
         foreach ( AliasAst* alias, node->names ) {
@@ -175,7 +213,7 @@ public:
     };
 
     // alias for exceptions (except FooBarException as somethingterriblehappened: ...)
-    virtual void visitExceptionHandler(ExceptionHandlerAst* node) override {
+    void visitExceptionHandler(ExceptionHandlerAst* node) override {
         AstDefaultVisitor::visitExceptionHandler(node);
         if ( ! node->name ) {
             return;
@@ -187,11 +225,31 @@ public:
         node->name->endCol = end;
     }
 
+    void visitString(Python::StringAst * node) override {
+        findString.indexIn(lines.at(node->startLine), node->startCol, QRegExp::CaretAtOffset);
+        if ( findString.matchedLength() > 0 ) {
+            node->endCol += findString.matchedLength();
+        }
+    }
+
+    void visitNumber(Python::NumberAst * node) override {
+        findNumber.indexIn(lines.at(node->startLine), node->startCol, QRegExp::CaretAtOffset);
+        if ( findNumber.matchedLength() > 0 ) {
+            node->endCol += findNumber.matchedLength();
+        }
+    }
+
+    void visitSubscript(Python::SubscriptAst * node) override {
+        AstDefaultVisitor::visitSubscript(node);
+        node->endCol = node->slice->endCol + 1; // Closing bracket (unless spaces...)
+    }
+
 private:
     const QStringList lines;
     QVector<KTextEditor::Cursor> dots;
     KTextEditor::Cursor attributeStart;
-
+    static const QRegExp findString;
+    static const QRegExp findNumber;
 
     // skip the decorators and the "def" at the beginning
     // of a class or function declaration and modify @arg node
@@ -359,25 +417,37 @@ private:
         return 0;
     };
 };
+// FIXME This only works for single-quoted (and oneline) strings.
+//  (otherwise it gives length 0 or 2, which is no worse than before).
+const QRegExp RangeFixVisitor::findString = QRegExp("^(?:\".*[^\\\\]\"|'.*[^\\\\]')");
+// Looser than the real spec, but since we know there *is* a valid number it finds the end ok.
+const QRegExp RangeFixVisitor::findNumber = QRegExp("^(?:[\\d_\\.bjoxBJOX]|[eE][+-]?)*");
 
 #include "generated.h"
 
 QMutex AstBuilder::pyInitLock;
 
 QString PyUnicodeObjectToQString(PyObject* obj) {
-#ifdef Q_OS_WIN32
-    // If you want to make this work on windows, take care to check Py_UNICODE_WIDE (see below)
-    // not sure if this always works
-    // not sure either why the "linux" version below wouldn't work on windows
-    return QString::fromWCharArray((wchar_t*)PyUnicode_AS_DATA(PyObject_Str(obj)));
-#else
-    uint* data = (uint*) PyUnicode_AS_DATA(PyObject_Str(obj));
-#ifdef Py_UNICODE_WIDE
-    return QString::fromUcs4(data);
-#else
-    return QString::fromUcs2(data);
-#endif // Py_UNICODE_WIDE
-#endif // windows
+    auto pyObjectCleanup = [](PyObject* o) { if (o) Py_DECREF(o); };
+    const auto strOwner = std::unique_ptr<PyObject, decltype(pyObjectCleanup)>(PyObject_Str(obj), pyObjectCleanup);
+    const auto str = strOwner.get();
+    if (PyUnicode_READY(str) < 0) {
+        qWarning("PyUnicode_READY(%p) returned false!", (void*)str);
+        return QString();
+    }
+    const auto length = PyUnicode_GET_LENGTH(str);
+    switch(PyUnicode_KIND(str)) {
+        case PyUnicode_1BYTE_KIND:
+            return QString::fromLatin1((const char*)PyUnicode_1BYTE_DATA(str), length);
+        case PyUnicode_2BYTE_KIND:
+            return QString::fromUtf16(PyUnicode_2BYTE_DATA(str), length);
+        case PyUnicode_4BYTE_KIND:
+            return QString::fromUcs4(PyUnicode_4BYTE_DATA(str), length);
+        case PyUnicode_WCHAR_KIND:
+            qWarning("PyUnicode_KIND(%p) returned PyUnicode_WCHAR_KIND, this should not happen!", (void*)str);
+            return QString::fromWCharArray(PyUnicode_AS_UNICODE(str), length);
+    }
+    Q_UNREACHABLE();
 }
 
 QPair<QString, int> fileHeaderHack(QString& contents, const QUrl& filename)
@@ -495,6 +565,12 @@ CodeAst::Ptr AstBuilder::parse(const QUrl& filename, QString &contents)
         
         PyErr_Fetch(&exception, &value, &backtrace);
         qCDebug(KDEV_PYTHON_PARSER) << "Error objects: " << exception << value << backtrace;
+
+        if ( ! value ) {
+            qCWarning(KDEV_PYTHON_PARSER) << "Internal parser error: exception value is null, aborting";
+            return CodeAst::Ptr();
+        }
+
         PyObject_Print(value, stderr, Py_PRINT_RAW);
         
         PyObject* errorMessage_str = PyTuple_GetItem(value, 0);
