@@ -26,6 +26,7 @@
 
 #include "parsesession.h"
 #include "pythoneditorintegrator.h"
+#include "kdevpythonversion.h"
 #include "declarationbuilder.h"
 #include "usebuilder.h"
 #include "astdefaultvisitor.h"
@@ -153,6 +154,10 @@ void PyAstTest::testStatements_data()
     QTest::newRow("continue") << "while True: continue";
     QTest::newRow("pass") << "pass";
     QTest::newRow("nonlocal") << "nonlocal x";
+#if PYTHON_VERSION_MAJOR >= 3 && PYTHON_VERSION_MINOR >= 6
+    QTest::newRow("varannotation1") << "primes: List[int] = []";
+    QTest::newRow("varannotation2") << "captain: str  # Note: no initial value!";
+#endif
 }
 
 void PyAstTest::testSlices()
@@ -221,6 +226,14 @@ void PyAstTest::testExpressions_data()
     QTest::newRow("None") << "None";
     QTest::newRow("False") << "False";
     QTest::newRow("True") << "True";
+
+#if PYTHON_VERSION_MAJOR >= 3 && PYTHON_VERSION_MINOR >= 6
+    QTest::newRow("async_generator") << "async def foo(): result = [i async for i in aiter() if i % 2]";
+    QTest::newRow("await_generator") << "async def foo(): result = [await fun() for fun in funcs]";
+
+    QTest::newRow("underscore_literals") << "0x_FF_FF_FF_FF";
+    QTest::newRow("formatted_string_literal") << "f\"He said his name is {name}.\"";
+#endif
     QTest::newRow("dstar_unpack") << "ext_map = {\n"
         "       **{ext: self.obj_extension for ext in self.src_extensions},\n"
         "       **{ext: self.res_extension for ext in self._rc_extensions + self._mc_extensions},\n"
