@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Copyright (c) 2007 Andreas Pakulat <apaku@gmx.de>                         *
  * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
- * Copyright (c) 2012 Sven Brauch <svenbrauch@gmail.com>                     *
+ * Copyright (c) 2012-2016 Sven Brauch <svenbrauch@gmail.com>                *
  *                                                                           *
  * This program is free software; you can redistribute it and/or             *
  * modify it under the terms of the GNU General Public License as            *
@@ -53,9 +53,13 @@
 #include "codegen/correctionfilegenerator.h"
 #include "kdevpythonversion.h"
 #include "pep8kcm/kcm_pep8.h"
+#include "projectconfig/projectconfigpage.h"
 #include "docfilekcm/kcm_docfiles.h"
+#include "pythonstylechecking.h"
+#include "helpers.h"
 
 #include <QDebug>
+#include <QProcess>
 #include "pythondebug.h"
 
 using namespace KDevelop;
@@ -84,10 +88,8 @@ LanguageSupport::LanguageSupport( QObject* parent, const QVariantList& /*args*/
     , KDevelop::ILanguageSupport()
     , m_highlighting( new Highlighting( this ) )
     , m_refactoring( new Refactoring( this ) )
+    , m_styleChecking( new StyleChecking( this ) )
 {
-    KDEV_USE_EXTENSION_INTERFACE( KDevelop::ILanguageSupport )
-    KDEV_USE_EXTENSION_INTERFACE( KDevelop::ILanguageCheckProvider )
-
     m_self = this;
 
     PythonCodeCompletionModel* codeCompletion = new PythonCodeCompletionModel(this);
@@ -108,9 +110,14 @@ void LanguageSupport::documentOpened(IDocument* doc)
     }
 
     DUChainReadLocker lock;
-    TopDUContextPointer topContext = TopDUContextPointer(DUChain::self()->chainForDocument(doc->url()));
+    ReferencedTopDUContext top = DUChain::self()->chainForDocument(doc->url());
     lock.unlock();
-    ParseJob::eventuallyDoPEP8Checking(IndexedString(doc->url()), topContext.data());
+    updateStyleChecking(top);
+}
+
+void LanguageSupport::updateStyleChecking(KDevelop::ReferencedTopDUContext top)
+{
+    m_styleChecking->updateStyleChecking(top);
 }
 
 LanguageSupport::~LanguageSupport()
@@ -150,7 +157,7 @@ SourceFormatterItemList LanguageSupport::sourceFormatterItems() const
     QString autopep8path = QStandardPaths::findExecutable("autopep8");
     if (autopep8path.isEmpty()) {
         // TODO: proper error handling/user notification
-        qWarning() << "Could not find the autopep8 executable";
+        qDebug() << "Could not find the autopep8 executable";
         autopep8path = "/usr/bin/autopep8";
     }
     autopep8.setContent(autopep8path + " -i $TMPFILE");
@@ -205,6 +212,18 @@ KDevelop::ConfigPage* LanguageSupport::configPage(int number, QWidget* parent)
     return nullptr;
 }
 
+int LanguageSupport::perProjectConfigPages() const
+{
+    return 1;
+}
+
+KDevelop::ConfigPage* LanguageSupport::perProjectConfigPage(int number, const KDevelop::ProjectConfigOptions& options, QWidget* parent)
+{
+    if ( number == 0 ) {
+        return new Python::ProjectConfigPage(this, options, parent);
+    }
+    return nullptr;
+}
 
 }
 
